# Make.pm
# implements 'new' and 'update' commands
#
# v0.44, 2004-06-15
# Copyright (c) Michel Messerschmidt 2002-2004
#
package Avtc::Make;
use strict;
use Exporter;
use File::Basename;
use File::Copy;

@Avtc::Make::ISA = qw(Exporter);
@Avtc::Make::EXPORT = qw(makeNew makeUpdate);


# creates direcories and lists for a new test
sub makeNew {
    if (not $::loadedGlobalConfig) {
        print("Command 'new' needs global configuration file\n");
        return 1;
    }
    my $ok = 0;
    # stores requested configuration options (this is user input!)
    my $name = "";
    my $finalName = "";
    my $testType = "normal";
    my $refName = "";
    my $mon3Name = "";
    my $mon6Name = "";
    my $syntax = $::configOs;

    if (not $::gui) {
        # testname
        my $nameOK = 0;
        while (not $nameOK) {
            ::log("Internal name of new test (used for directory names, ".
                  "max. 8 characters):\n");
            $name = <STDIN>;
            chomp($name);
            ::log("\n");
            if (length($name) > 8) {
                $name = substr($name, 0, 8);
                ::log("Input too long: name cut to '$name'\n");
            }
            $name = lc($name);
            if ($name eq "") {
                ::log("Illegal name: empty name '' is not acceptable\n");
            } elsif ($name eq "all") {
                ::log("Illegal name: 'all' is a reserved name\n");
            } elsif (exists $::configAllTests{$name}) {
                ::log("Illegal name: a test named '$name' exists already\n");
            } else {
                $nameOK = 1;
            }
        }
        # now the final testname
        ::log("Final name of new test (used for test report, ".
              "can be changed later):\n");
        $finalName = <STDIN>;
        chomp($finalName);
        ::log("\n");
        # now the testtype options
        ::log("\nIs this a Heureka Test (y/n) ?   ");
        $testType = <STDIN>;
        chop($testType);
        if (lc($testType) eq "y")  {
            $testType = "heureka";
            my $refOK = 0;
            while (not $refOK) {
                ::log("\nEnter internal name of Reference Test: ");
                $refName = <STDIN>;
                chop($refName);
                if (exists $::configAllTests{$refName}) {
                    $refOK = 1;
                } else {
                    ::log("Unknown reference test name\n");
                }
            }
            my $ref3OK = 0;
            while (not $ref3OK) {
                ::log("\nEnter name of first 3-month period (3 characters): ");
                $mon3Name = <STDIN>;
                chop($mon3Name);
                validate3Char($mon3Name);
                $ref3OK = 1 unless ($mon3Name eq "");
            }
            my $ref6OK = 0;
            while (not $ref6OK) {
                ::log("\nEnter name of second 3-month period (3 characters): ");
                $mon6Name = <STDIN>;
                chop($mon6Name);
                validate3Char($mon6Name);
                $ref6OK = 1 unless ($mon6Name eq "");
            }
        } else {
            $testType = "normal";
        }

        # and the syntax option
        ::log("\nWhich syntax should be used for the testbeds (this also \n".
              "affects how the dirlists must be created):\n");
        ::log("  w = dos/windows syntax\n  u = unix syntax");
        while ($syntax ne "w" and $syntax ne "u") {
            ::log("\n  ? ");
            $syntax = <STDIN>;
            $syntax = lc(::chommp($syntax));
            if (length($syntax) > 1) {
                $syntax = substr($syntax, 0, 1);
                ::log("Input too long: cut to '$syntax'\n");
            }
        }
        ::log("\n\n");
        if ($syntax eq "w") {
            ::log("\nTest uses Dos/Windows syntax\n");
            $syntax = $::OS_WINDOWS;
        } else {
            ::log("\nTest uses Unix syntax\n");
            $syntax = $::OS_UNIX;
        }
        ::log("Internal testname is '$name'\n");
        ::log("Final testname is '$finalName'\n");
        if ($testType eq "heureka") {
            ::log("This is a heureka test\n");
            ::log("Reference test is '$refName'\n");
        } else {
            ::log("This is a normal test\n");
        }
        ::log("\nProceed with directory creation (y/n) ?   ");
        my $tempAnswer = <STDIN>;
        chomp($tempAnswer);
        if (lc($tempAnswer) eq "y")  {
            $ok = 1;
        }

    } else {
        # and now the GUI version
        my(@ll) = qw/-anchor e -justify right/;
#        my(@el) = qw/-anchor w -justify left -side left -relief sunken -padx 10 -pady 5 -fill x/;
        my(@el) = qw/-justify left -relief sunken/;
#        my(@rl) = qw/-anchor w -justify left -padx 10 -pady 5/;
        my(@rl) = qw/-anchor w -justify left/;
        my $dlg = $::mainWin->DialogBox(-title => "Setup new test",
                                   -buttons => ["Ok", "Cancel"],
                                   -default_button => "Cancel");
        my $l1 = $dlg->add("Label", @ll, -text => "Internal name of new test (max. 8 characters):")
            ->pack();
        my $w1 = $dlg->add("Entry", @el, -width => 20,
                                    -textvariable => \$name)
            ->pack();
        my $l2 = $dlg->add("Label", @ll, -text => "Final name of new test:")
            ->pack();
        my $w2 = $dlg->add("Entry", @el, -width => 40,
                                    -textvariable => \$finalName)
            ->pack();
        my $l3 = $dlg->add("Label", @ll, -text => "Is this a Heureka Test:")
            ->pack();
        my $r3a = $dlg->add("Radiobutton", @rl, -text => "yes",
                                           -value => "heureka",
                                           -variable => \$testType)
            ->pack();
        my $r3b = $dlg->add("Radiobutton", @rl, -text => "no",
                                           -value => "normal",
                                           -variable => \$testType)
            ->pack();
        my $l4 = $dlg->add("Label", @ll, -text => "Syntax used in this test:")
            ->pack();
        my $r4a = $dlg->add("Radiobutton", @rl, -text => "Windows",
                                           -value => $::OS_WINDOWS,
                                           -variable => \$syntax)
            ->pack();
        my $r4b = $dlg->add("Radiobutton", @rl, -text => "Unix",
                                           -value => $::OS_UNIX,
                                           -variable => \$syntax)
            ->pack();
        # show dialog
        my $button = $dlg->Show;
        if ($button eq "Ok") {
            $ok = 1;
        }
        # validate input
        if (length($name) > 8) {
            $name = substr($name, 0, 8);
            ::log("Input too long: name cut to '$name'\n");
        }
        $name = lc($name);
        if ($name eq "") {
            ::log("Illegal name: empty name '' is not acceptable\n");
            $ok = 0;
        } elsif ($name eq "all") {
            ::log("Illegal name: 'all' is a reserved name\n");
            $ok = 0;
        } elsif (exists $::configAllTests{$name}) {
            ::log("Illegal name: a test named '$name' exists already\n");
            $ok = 0;
        }
        # create second dialog to get heureka options
        if ($ok and $testType eq "heureka") {
            $ok = 0;
            my $dlgH = $::mainWin->DialogBox(-title => "Setup new test - Heureka settings",
                                       -buttons => ["Ok", "Cancel"],
                                       -default_button => "Cancel");
            my $lh1 = $dlgH->add("Label", @ll, -text => "Name reference test:")
                ->pack();
            foreach my $i (sort keys %::configAllTests) {
                $dlgH->add("Radiobutton", @rl, -text => $i,
                                          -value => $i,
                                          -variable => \$refName)
                     ->pack();
            }
            my $lh2 = $dlgH->add("Label", @ll, -text => "Name of first 3-month period (3 characters):")
                ->pack();
            my $wh2 = $dlgH->add("Entry", @el, -width => 5,
                                        -textvariable => \$mon3Name)
                ->pack();
            my $lh3 = $dlgH->add("Label", @ll, -text => "Name of second 3-month period (3 characters):")
                ->pack();
            my $wh3 = $dlgH->add("Entry", @el, -width => 5,
                                        -textvariable => \$mon6Name)
                ->pack();

            # show dialog
            my $button = $dlgH->Show;
            if ($button eq "Ok") {
                $ok = 1;
            }
            if (not exists $::configAllTests{$refName}) {
                $ok = 0;
                ::log("Unknown reference test name\n");
            }
            validate3Char($mon3Name);
            validate3Char($mon6Name);
            if ($mon3Name eq "" or $mon6Name eq "") {
                $ok = 0;
            }
        }

        # create confirmation dialog
        if ($ok) {
            $ok = 0;
            my $dlgC = $::mainWin->DialogBox(-title => "Confirm new test setup",
                                       -buttons => ["Yes", "No"],
                                       -default_button => "No");
            my $lc1 = $dlgC->add("Label", @ll, -text => "Internal name of new test: $name")
                ->pack();
            my $lc2 = $dlgC->add("Label", @ll, -text => "Final name of new test: $finalName")
                ->pack();
            my $lc3 = $dlgC->add("Label", @ll, -text => "Test uses '$syntax' syntax")
                ->pack();
            if ($testType eq "heureka") {
                my $lc4 = $dlgC->add("Label", @ll, -text => "This is a heureka test")
                    ->pack();
                my $lc4a = $dlgC->add("Label", @ll, -text => "Reference test is: $refName")
                    ->pack();
            } else {
                my $lc4 = $dlgC->add("Label", @ll, -text => "This is a normal test")
                    ->pack();
            }
            my $lc5 = $dlgC->add("Label", @ll, -text => "\n\nCreate new test ?")
                ->pack();
            # show dialog
            my $button = $dlgC->Show;
            if ($button eq "Yes") {
                $ok = 1;
            }
        }
    }


    if (not $ok)  {
        ::log("...command cancelled.\n");
        return 1;
    }

    ::log("...creating base directories\n");
    # this is the absolute path in current syntax
    my $configFile = $::configRoot{$::configOs} . $::sep .
                     $::configDirNames{"eval"} . $::sep .
                     $name . $::sep . $name . ".cfg";
    # make sure that root directories exist
    ::makeDir($::LogRoot) or do {
        ::log("ERROR: creating directory $::LogRoot\n");
        return 1;
    };
    ::makeDir($::NewLogRoot) or do {
        ::log("ERROR: creating directory $::NewLogRoot\n");
        return 1;
    };
    ::makeDir($::EvalRoot) or do {
        ::log("ERROR: creating directory $::EvalRoot\n");
        return 1;
    };
    ::makeDir($::InfoRoot) or do {
        ::log("ERROR: creating directory $::InfoRoot\n");
        return 1;
    };
    # create subdirectories for the new test
    my $dir = $::LogRoot . $::sep . $name;
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::NewLogRoot . $::sep . $name;
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name;
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"lists"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"keys"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"result"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"resultCSV"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"resultLossCSV"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };
    $dir = $::EvalRoot . $::sep . $name . $::sep . $::configDirNames{"report"};
    ::makeDir($dir) or do {
        ::log("ERROR: creating directory $dir\n");
        return 1;
    };

    # now copy all template scanner keys and lists to new directories
    my @files = ();
    my $file;
    # the lists first ...
    $dir = $::EvalRoot . $::sep . $::configDirNames{"template"} . $::sep .
           $::configDirNames{"lists"};
    opendir(DIR, $dir) or do {
        ::log("  ERROR reading directory $dir\n");
        return 1;
    };
    my $filename;
    while (defined($filename = readdir(DIR))) {
        # Most lists have a .LST extension
        # For packed testbeds there is an additional .PAK list
        # .ANO are list with "anomaly files" 
        # .FP are lists with regualr expressions for "false positives"
        if ($filename =~ /\.(LST|PAK|FP|ANO)$/i) {
            push @files, $filename;
        }
    }
    my $destDir = $::EvalRoot.$::sep.$name.$::sep.$::configDirNames{"lists"};
    foreach $file (@files) {
        copy($dir.$::sep.$file, $destDir.$::sep.$file) or do {
            ::log("  ERROR copying template file $file\n");
            return 1;
        };
    }
    # ... and then the scanner keys
    @files = ();
    $dir = $::EvalRoot . $::sep . $::configDirNames{"template"} . $::sep .
           $::configDirNames{"keys"};
    opendir(DIR, $dir) or do {
        ::log("  ERROR reading directory $dir\n");
        return 1;
    };
    while (defined($filename = readdir(DIR))) {
        if ($filename =~ /\.(PL|AWK)$/i) {
            push @files, $filename;
        }
    }
    $destDir = $::EvalRoot.$::sep.$name.$::sep.$::configDirNames{"keys"};
    foreach $file (@files) {
        copy($dir.$::sep.$file, $destDir.$::sep.$file) or do {
        ::log("  ERROR copying scanner key $file\n");
        return 1;
        };
    }

    # create the test-specific configuration from a template file
    my $templateConfig = $::EvalRoot . $::sep . $::configDirNames{"template"} .
                         $::sep . $::configOs . ".cfg";
    open(TEMPLATE, "< $templateConfig") or do {
        ::log("  ERROR reading template configuration file ".
              $templateConfig."\n");
        return 1;
    };
    binmode(TEMPLATE);
    open(CONFIG, "> $configFile") or do {
        ::log("  ERROR creating test-specific configuration file ".
              $configFile."\n");
        return 1;
    };
    binmode(CONFIG);
    my $line;
    while (defined($line = <TEMPLATE>)) {
        $line = ::chommp($line);
        if ($line =~ /^\$\:\:TestName\s*\=/) {
            $line = '$::TestName = "' . $finalName . '";';
        }
        elsif ($line =~ /^\$\:\:configSyntax\s*\=/) {
            $line = '$::configSyntax = "' . $syntax . '";';
        }
        elsif ($line =~ /^\$\:\:TestType\s*\=/) {
            $line = '$::TestType = "' . $testType . '";';
        }
        elsif ($line =~ /^\$\:\:RefTestName\s*\=/) {
            $line = '$::RefTestName = "' . $refName . '";';
        }
        elsif ($line =~ /^\$\:\:TestMonth3Name\s*\=/) {
            $line = '$::TestMonth3Name = "' . $mon3Name . '";';
        }
        elsif ($line =~ /^\$\:\:TestMonth6Name\s*\=/) {
            $line = '$::TestMonth6Name = "' . $mon6Name . '";';
        }
        print(CONFIG $line, $::EOL);
    }
    close(TEMPLATE);
    close(CONFIG);

    # create entry for the new test in global configuration file
    open(GLOBAL, ">> $::configGlobalFile") or do {
        ::log("ERROR writing to global configuration file !!!\n");
        return 1;
    };
    binmode(GLOBAL);
    print(GLOBAL $::EOL,"[",$name,"]",$::EOL);
    print(GLOBAL $::OS_UNIX," = ",
                 $::configRoot{$::OS_UNIX} . $::sep_UNIX .
                 $::configDirNames{"eval"} . $::sep_UNIX . $name .
                 $::sep_UNIX . $name . ".cfg", $::EOL);
    print(GLOBAL $::OS_WINDOWS," = ",
                 $::configRoot{$::OS_WINDOWS} . $::sep_WINDOWS .
                 $::configDirNames{"eval"} . $::sep_WINDOWS . $name .
                 $::sep_WINDOWS . $name . ".cfg", $::EOL, $::EOL);
    close(GLOBAL);

    # add new test to test list (just for the currently running session)
    $::configAllTests{$name} = $configFile;

    ::log("New test setup finished.\n".
          "Now edit the lists in the 'LISTS' directory and run\n".
          "  avtctest update\n".
          "afterwards to complete the test setup.\n\n".
          "If the new test should be the default test, change the \n".
          "corresponding line in the global configuration file.\n\n");

    return 0;
}


sub validate3Char {
    my $in = $_[0];
    if (length($in) > 3) {
        $in = substr($in, 0, 3);
        ::log("Long input cut to '$in'\n");
    } elsif (length($in) == 2) {
        $in .= "_";
        ::log("Short input modified to '$in'\n");
    } elsif (length($in) == 1) {
        $in .= "__";
        ::log("Short input modified to '$in'\n");
    } else {
        ::log("ERROR: empty input\n");
    }
    return $in;
}


sub makeUpdate {
    my $ret = 0;
    if (not defined($::DirListDir) or not -d $::DirListDir) {
        ::log("ERROR: dirlist directory not found\n", 0);
        return 1;
    }
    ::log("...creating  subdirectories\n");
    makeDirs();
    my $tb;
    # this must be done in sorted order because the packed testbeds need
    # the lists from the unpacked testbed ("_" must be processed before all
    # other characters (currently P,Q,R,S,E)
    LOOP_TB: foreach $tb (reverse sort keys %::lists) {
        my $tbId = $::lists{$tb}->{"id"};
        my $tbDir = $::lists{$tb}->{"path"}->{$::configOs};
        my $dirFile = $::ListDir . $::sep . $::FilePreDirlist . $tbId . "." .
                      $::FileExtLists;
        my $fpFile = $::ListDir . $::sep . $tb."." . $::FileExtFP;

        # if there are false positives, ALL evallists must be filtered
        if (exists($::lists{$tb}->{"fp"})) {
            if ($::debug) {
                ::log("This testbed includes false positives\n");
            }
            # create dirlist (used for all evallists belonging to this testbed)
            ::log("creating dirlist for $tb - $tbDir\n");
            my $inList = $::DirListDir . $::sep . $::FilePreDirlist . $tbId . "." .
                          $::FileExtText;
            makeDirList($inList, $dirFile) or do {
                ::log("ERROR: update of dirlist for $tbDir failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };

            my $fpList = $::lists{$tb}->{"fp"};
            my $sampleListFile = $::ListDir . $::sep . $tb . "." .
                                 $::FileExtSamples;
            my $fpSampleListFile = $::ListDir . $::sep . $fpList . "." .
                                   $::FileExtSamples;
            makeEvalFP($tb, $dirFile, $fpFile, $sampleListFile,
                       $fpSampleListFile) or do {
                ::log("ERROR: update of evaluations lists for $tb - ".$tbDir.
                      " failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };
        } elsif (exists($::lists{$tb}->{"packed"})) {
            if ($::debug) {
                ::log("This is a packed testbed \n");
            }
            # create dirlist (used for all evallists belonging to this testbed)
            ::log("creating dirlist for $tb - $tbDir\n");
            my $inList = $::DirListDir . $::sep . $::FilePreDirlist . $tbId . "." .
                          $::FileExtText;
            makeDirListPacked($tb, $inList, $dirFile) or do {
                ::log("ERROR: update of dirlist for $tbDir failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };

            # create dirlist for archive contents
            makeArchiveDirList($tb) or do {
                ::log("ERROR: update of dirlist $tbDir failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };
            # packed testbeds must be handled different
            makeEvalPacked($tb, $dirFile) or do {
                ::log("ERROR: update of evaluations lists for ".$tbDir.
                      " failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };
        } else {
            # most testbeds will be of this type
            # create dirlist (used for all evallists belonging to this testbed)
            ::log("creating dirlist for $tb - $tbDir\n");
            my $inList = $::DirListDir . $::sep . $::FilePreDirlist . $tbId . "." .
                          $::FileExtText;
            makeDirList($inList, $dirFile) or do {
                ::log("ERROR: update of dirlist for $tbDir failed !!!\n\n");
                $ret = 1;
                next LOOP_TB;
            };

            my $evalList;
            foreach $evalList (keys %{$::lists{$tb}->{"evals"}}) {
                ::log("creating evaluation list $evalList for testbed ".
                      $tb."\n");
                my $anoRef = \%{$::lists{$tb}->{"evals"}->{$evalList}->{"ano"}};
#                my $evalListFile = $::ListDir . $::sep . $evalList . ".vir";
                my $virListFile = $::ListDir . $::sep . $evalList;
                makeEval($dirFile, $virListFile, $anoRef) or do {
                    ::log("ERROR: update of evaluation list ".$evalList.
                          " failed !!!\n\n");
                    $ret = 1;
                    next;
                };
                ::log("creating summary of $evalList for $tb\n");
                makeSum($virListFile) or do {
                    ::log("ERROR: update of summary ".$evalList.".".
                          $::FileExtSummary." failed !!!\n\n");
                    $ret = 1;
                    next;
                };
            }
        }
    }  #end of LOOP_TB

    return $ret;
}


########################## update subroutines ##############################

sub makeDirs {
    # create all subdirectories, if they don't already exist
    my ($os, $tb, $dir);
    foreach $os (keys %::osListAll) {
        $dir = $::EvalRoot . $::sep . $::configTestName . $::sep . $os;
        ::makeDir($dir) or do {
            ::log("ERROR creating directory $dir\n");
        };
        $dir = $::LogRoot . $::sep . $::configTestName . $::sep . $os;
        ::makeDir($dir) or do {
            ::log("ERROR creating directory $dir\n");
        };
        $dir = $::NewLogRoot . $::sep . $::configTestName . $::sep . $os;
        ::makeDir($dir) or do {
            ::log("ERROR creating directory $dir\n");
        };
        foreach $tb (keys %::listsAll) {
            $dir = $::EvalRoot . $::sep . $::configTestName . $::sep . $os .
                   $::sep . $tb;
            ::makeDir($dir) or do {
                ::log("ERROR creating directory $dir\n");
            };
            $dir = $::LogRoot . $::sep . $::configTestName . $::sep . $os .
                   $::sep . $tb;
            ::makeDir($dir) or do {
                ::log("ERROR creating directory $dir\n");
            };
# currently the driveletter is expected by many scripts, but this may change!!
            $dir = $::NewLogRoot . $::sep . $::configTestName . $::sep . $os .
                   $::sep . $::listsAll{$tb}->{"id"};
#                   $::sep . $tb;
            ::makeDir($dir) or do {
                ::log("ERROR creating directory $dir\n");
            };
        }
    }

    #create initial statusfile for each product/tb/os
    ::log("...setting initial product status\n");
    my $newStatus="-S1";
    my ($prod, $file);
    foreach $os (keys %::osListAll) {
        foreach $tb (keys %::listsAll) {
            foreach $prod (keys %::prodListAll) {
                $file = $::EvalRoot . $::sep . $::configTestName . $::sep . $os .
                        $::sep . $tb . $::sep . $prod . $::FileExtStatus;
                # don't overwrite any existing status file !!!
                if (not -e $file) {
                    open(STAT, "> $file") or do {
                        ::log("ERROR: can't create statusfile $file\n");
                    };
                    binmode(STAT);
                    print(STAT $newStatus, $::EOL);
                    close(STAT);
                }
            }
        }
    }
}


# this creates a new dirlist
sub makeDirList {
    my $inDirList= $_[0];
    my $outDirList = $_[1];

    ::makeDir($::TempBase);

    # part 1 - make all uppercase (but only for windows syntax)
    my $in = $inDirList;
    my $temp = $::TempBase . $::sep . "dir_list.tmp";
    open(IN, "< $in") or do {
        ::log("ERROR opening $in: $!\n");
        return 0;
    };
    binmode(IN);
    open(OUT, "> $temp") or do {
        ::log("ERROR opening $temp: $!\n");
        close(IN);
        return 0;
    };
    binmode(OUT);

    my $line;
    while (defined($line = <IN>)) {
        # remove all trailing control characters
        $line = ::chommp($line);
        # remove all preceding and trailing whitespace characters
        $line =~ s/^\s*(\S+)\s*$/$1/;
        # windws syntax is case-insensitive, so make all uppercase
        # to avoid problems when comparing/sorting lines
        if ($::configSyntax eq $::OS_WINDOWS) {
            $line =  uc($line);
        }
        print(OUT $line, $::EOL);
    }
    close(IN);
    close(OUT);

    # part 2 - sort
    ::mysort($temp, $outDirList, 0) or do {
        return 0;
    };

    unlink($temp);
    return 1;
}


# this creates a new dirlist
sub makeDirListPacked {
    my $tb = $_[0];
    my $inDirList= $_[1];
    my $outDirList = $_[2];

    ::makeDir($::TempBase);

    # part 1 - make all uppercase (but only for windows syntax)
    my $in = $inDirList;
    my $temp = $::TempBase . $::sep . "dir_list.tmp";
    open(IN, "< $in") or do {
        ::log("ERROR opening $in: $!\n");
        return 0;
    };
    binmode(IN);
    open(OUT, "> $temp") or do {
        ::log("ERROR opening $temp: $!\n");
        close(IN);
        return 0;
    };
    binmode(OUT);

    my $line;
    my $lastLine = "";
    while (defined($line = <IN>)) {
        # remove all trailing control characters
        $line = ::chommp($line);
        # remove all preceding and trailing whitespace characters
        $line =~ s/^\s*(\S+)\s*$/$1/;
        # windws syntax is case-insensitive, so make all uppercase
        # to avoid problems when comparing/sorting lines
        if ($::configSyntax eq $::OS_WINDOWS) {
            $line =  uc($line);
        }
        # test for invalid dirlist entries and entries that must be modified
        my $checked = 0;
        my $pack;
        # only loop over the compression programs that are actually used
        # for this testbed
        LOOP_PRG: foreach $pack (sort keys %{$::listsAll{$tb}->{"packed"}->{"prg"}}) {
            my $packFile = $::listsAll{$tb}->{"packed"}->{"prg"}->{$pack}->{"filename"};
            if ($::debug > 1) {
                ::log("    test match of packFile '".$packFile.
                      "' in dirlist entry '$line'\n");
            }
            if ($line =~ /\Q$packFile\E/i) {
                # this is a known filename for a defined compression program
                $checked = 1;
                # some packed testbeds have compression programs that can't create
                # archives. These programs have an additional subdirectory with
                # single compressed files in it, that must not appear in the dirlist
                my $cut = ($line =~ s/^(.*\Q$packFile\E)(.+)$/$1/i);
                if ($::debug and $cut) {
                    ::log("DEBUG:    cut '".$2."' from dirlist entry '".
                          $line."'\n");
                }
                last LOOP_PRG;
            }
        }

        if ($checked) {
            if ($line ne $lastLine) {
                print(OUT $line, $::EOL);
                $lastLine = $line;
            }
        } else {
            ::log("ERROR creating archive dirlist: found entry without ".
                  "known filename for a compression program\n");
            ::log("ERROR: creation of archive dirlist failed!\n");
            close(IN);
            close(OUT);
            return 0;
        }
    }
    close(IN);
    close(OUT);

    # part 2 - sort
    ::mysort($temp, $outDirList, 0) or do {
        return 0;
    };

    unlink($temp);
    return 1;
}


# this creates a new dirlist
sub makeArchiveDirList {
    my $tb = $_[0];
    my $tbId = $::listsAll{$tb}->{"id"};
    my $tbDir = $::listsAll{$tb}->{"path"}->{$::configOs};
    # find corresponding uncompressed testbed
    my $unpackedTb = $tb;
    substr($unpackedTb, 3, 2) = "__";
    my $unpackedTbId = $::listsAll{$unpackedTb}->{"id"};
    my $unpackedTbDir = $::listsAll{$unpackedTb}->{"path"}->{$::configOs};
    my $inList = $::ListDir . $::sep . $::FilePreDirlist . $unpackedTbId .
                 "." . $::FileExtLists;
    my $outList = $::ListDir . $::sep . $::FilePreDirlist . $tbId .
                  $::FileExtArchiveFiles . "." . $::FileExtLists;

    ::log("creating dirlist for $tb - archive contents\n");
    open(IN_DIRLST, "< $inList") or do {
        ::log("ERROR opening $inList: $!\n");
        return 0;
    };
    binmode(IN_DIRLST);
    open(OUT_DIRLST, "> $outList") or do {
        ::log("ERROR opening $outList: $!\n");
        close(IN_DIRLST);
        return 0;
    };
    binmode(OUT_DIRLST);

    my $line;
    while(defined($line = <IN_DIRLST>)) {
        $line = ::chommp($line);
### old code relied on drive letters
#        substr($line, 0, 1) = $tbDir;
        $line =~ s/^\Q$unpackedTbDir\E(.*)$/$tbDir$1/;
        print(OUT_DIRLST $line, $::EOL);
    }

    close(IN_DIRLST);
    close(OUT_DIRLST);
    return 1;
}


# this creates the summary
sub makeSum {
    my $in = $_[0];     # evaluation list (viruses) to summarize
    my $out = $in . "." . $::FileExtSummary;
    my $line = "";      # current line from input file
    my $samples = 0;    # counts nr. of samples/files in variantlist
    my $newSamples = 0; # nr. of samples/files in current line/directory
    my $viruses = 0;    # counts nr. of viruses in variantlist
    my $path = "";      # just to split the line - data is not needed
    my $lastPath = "";  # only needed for consistency check
    my @garbage = ();   # collect remaining words in variantlist (should never
                        # occur)
    open(IN, "< $in") or do {
        ::log("ERROR opening $in: $!\n");
        close(IN);
        return 0;
    };
    binmode(IN);
    open(OUT, "> $out") or do {
        ::log("ERROR opening $out: $!\n");
        close(IN);
        close(OUT);
        return 0;
    };
    binmode(OUT);

    while (defined($line = <IN>)) {
        $line = ::chommp($line);
        ($path, $newSamples, @garbage) = split(" ", $line);
        # some format checking
        if (($path eq $lastPath) or ($newSamples < 1)) {
            ::log("ERROR in variantlist found\n");
            return 0;
        }
        $viruses++;
        $samples += $newSamples;
        $lastPath = $path;
    }

    printf(OUT "Testbed  %6d 100.0%%                        %6d 100.0%%%s",
               $viruses, $samples, $::EOL);

    close(IN);
    close(OUT);
    return 1;
}


# this creates a new evaluation list
sub makeEval {
    my $in = $_[0];      # input list of all files (for this evaluation)
    my $out = $_[1];     # output list of all viruses and the nr. of their
                         # files (for this evaluation)
    my %anoList = %{$_[2]}; # all anomalies (files & nr), given by reference
    my $samples = 0;     # counts nr. of samples/files from a virus-variant/dir
    my $line = "";       # file name with full path of current sample
    my $path = "";       # only the path (= name of this virus variant)
    my $fileName = "";   # only file name of current sample
    my $lastPath = "";
    my $lastFileName = "";
    # set Windows directory syntax for logfile parsing cause all logfiles must
    # have windows syntax
    if ($::configSyntax eq $::OS_WINDOWS) {
    fileparse_set_fstype("MSWin32");
    } else {
    fileparse_set_fstype("Unix");
    }

    # count files
    open(IN, "< $in") or do {
        ::log("ERROR opening $in: $!\n");
        close(IN);
        return 0;
    };
    binmode(IN);
    open(OUT, "> $out") or do {
        ::log("ERROR opening $out: $!\n");
        close(IN);
        close(OUT);
        return 0;
    };
    binmode(OUT);

    my $firstLine = 1;
    # get all lines
    LOOP_MK_IN: while (defined($line = <IN>)) {
        my $ano;
        $line = ::chommp($line);
        ($fileName, $path) = fileparse($line);
        chop($path);
        if ($firstLine) {
            # tests if input file is empty ("0" will never be a valid dirname)
            if (not $line) {
                # no input -> no output (don't write anything);
                ::log("WARNING: first line is empty in file $in\n");
                close(IN);
                close(OUT);
                return 0;
            }
            # filter anomaly files before counting this sample
            # since this is the first sample of a virus, it is only filtered if
            # the maximum number of counted samples for this virus is 0
            foreach $ano (keys %anoList) {
                if (($line =~ /^\Q$ano\E/)
                    and (0 >= $anoList{$ano})) {
                    if ($::debug) {
                        ::log("filtered anomaly file: $line\n");
                    }
                    # this file is not counted as sample
                    next LOOP_MK_IN;
                }
            }
            $firstLine = 0;
            $samples = 1;
            $lastPath = $path;
            $lastFileName = $fileName;
            next LOOP_MK_IN;
        }

        if ($path eq $lastPath) {
            # it's the same virus-variant
            if ($fileName eq $lastFileName) {
                # it's the same sample
                ::log("ERROR: identical lines in dirlist\n");
                close(IN);
                close(OUT);
                return 0;
            } else {
                # it's another sample of the same virus-variant
                # filter anomaly files before counting this sample
                foreach $ano (keys %anoList) {
                    if (($line =~ /^\Q$ano\E/)
                        and ($samples >= $anoList{$ano})) {
                        if ($::debug) {
                            ::log("filtered anomaly file: $line\n");
                        }
                        # this file is not counted as sample
                        next LOOP_MK_IN;
                    }
                }
                $lastFileName = $fileName;
                $samples++;
            }
        } else {
            # it's the next virus-variant
            # filter anomaly files before counting this sample
            foreach $ano (keys %anoList) {
                # since this is the first sample of a virus, it is only
        # filtered if the maximum number of counted samples for
        # this virus is 0
                if (($line =~ /^\Q$ano\E/)
                    and (0 >= $anoList{$ano})) {
                    if ($::debug) {
                        ::log("filtered anomaly file: $line\n");
                    }
                    # this file is not counted as sample
                    next LOOP_MK_IN;
                }
            }
            printf(OUT "%s %d%s",$lastPath,$samples,$::EOL);
            $lastPath = $path;
            $lastFileName = $fileName;
            $samples = 1;
        }
    }
    # don't forget the last line / virus-variant
    printf(OUT "%s %d%s",$lastPath,$samples,$::EOL);
    close(IN);
    close(OUT);
    return 1;

}


# create evallists for false positive testbeds
sub makeEvalFP {
    my $tb = $_[0];               # testbed identifier (type)
    my $dirFile = $_[1];          # input list of all files in this testbed
    my $fpFile = $_[2];           # list of pattern that describe all false
                                  # positive filenames in this testbed
    my $sampleListFile = $_[3];   # output list of all malware samples
    my $fpSampleListFile = $_[4]; # output list of all false positive samples

    # split dirlist in two parts: all false positives vs. all malware
    my @fpList = ();
    open(FPLIST, "< $fpFile") or do {
        ::log("ERROR opening $fpFile: $!\n");
    };
    binmode(FPLIST);

    my $inFP;
    while (defined($inFP = <FPLIST>)) {
        $inFP = ::chommp($inFP);
    if ($::configSyntax eq $::OS_WINDOWS) {
        # must be uppercase because the windows dirlist is in uppercase too
        $inFP = uc($inFP);
    }
        push @fpList, $inFP;
        if ($::debug) {
            ::log("added fp = '$inFP'\n");
        }
    }
    close(FPLIST);
    open(DIRLIST, "< $dirFile") or do {
        ::log("ERROR opening $dirFile: $!\n");
        return 0;
    };
    binmode(DIRLIST);
    open(SAMLIST, "> $sampleListFile") or do {
        ::log("ERROR opening $sampleListFile: $!\n");
        close(DIRLIST);
        return 0;
    };
    binmode(SAMLIST);
    open(SAMLIST_FP, "> $fpSampleListFile") or do {
        ::log("ERROR opening $fpSampleListFile: $!\n");
        close(DIRLIST);
        close(SAMLIST);
        return 0;
    };
    binmode(SAMLIST_FP);

    my ($line, $fp);

    if ($::debug) {
        ::log("starting fp separation...\n");
    }
    LOOP_SPLIT: while(defined($line = <DIRLIST>)) {
        $line = ::chommp($line);
        foreach $fp (@fpList) {
            if ($line =~ /\Q$fp\E/) {
                print(SAMLIST_FP $line, $::EOL);
                next LOOP_SPLIT;
            }
        }
        print(SAMLIST $line, $::EOL);
    }
    close(DIRLIST);
    close(SAMLIST);
    close(SAMLIST_FP);

    my $evalList;
    foreach $evalList (keys %{$::lists{$tb}->{"evals"}}) {
        ::log("creating evaluation list $evalList for testbed $tb\n");
        my $anoRef = \%{$::lists{$tb}->{"evals"}->{$evalList}->{"ano"}};
        my $virListFile = $::ListDir . $::sep . $evalList;
        if ($::lists{$tb}->{"evals"}->{$evalList}->{"id"} eq "F") {
            makeEval($fpSampleListFile, $virListFile, $anoRef) or do {
                ::log("ERROR: update of evaluation list $evalList failed ".
                      "!!!\n\n");
                next;
            };
        } else {
            makeEval($sampleListFile, $virListFile, $anoRef) or do {
                ::log("ERROR: update of evaluation list ".$evalList.
                      " failed !!!\n\n");
                next;
            };
        }
        # create summary
        ::log("creating summary of $evalList for $tb\n");
        makeSum($virListFile) or do {
            ::log("ERROR: update of summary ".$evalList.
                  ".sum failed !!!\n\n");
            next;
        };
    }

    return 1;
}


# create evallists for packed testbeds
sub makeEvalPacked {
    my $tb = $_[0];
    my $dirFile = $_[1];
    my $tbDir = $::listsAll{$tb}->{"path"}->{$::configOs};
    # find corresponding uncompressed testbed
    my $unpackedTb = $tb;
    substr($unpackedTb, 3, 2) = "__";
    my $unpackedTbDir = $::listsAll{$unpackedTb}->{"path"}->{$::configOs};
    my $inList = $::ListDir . $::sep . $unpackedTb;
    my $outList = $::ListDir . $::sep . $tb;

    # create evaluation list for archive contents
    ::log("creating evaluation list $tb for testbed $tb\n");
    open(INLIST, "< $inList") or do {
        ::log("ERROR opening $inList: $!\n");
        return 0;
    };
    binmode(INLIST);
    open(OUTLIST, "> $outList") or do {
        ::log("ERROR opening $outList: $!\n");
        close(INLIST);
        return 0;
    };
    binmode(OUTLIST);

    my $line;
    while(defined($line = <INLIST>)) {
        $line = ::chommp($line);
    $line =~ s/^\Q$unpackedTbDir\E(.*)$/$tbDir$1/;
        print(OUTLIST $line, $::EOL);
    }
    close(INLIST);
    close(OUTLIST);

    ::log("creating summary of $tb for $tb\n");
    my $ok = copy($::ListDir . $::sep . $unpackedTb . "." . $::FileExtSummary,
                  $::ListDir . $::sep . $tb . "." . $::FileExtSummary);


    # create evaluation list for archives themselves
    my $evalList = $tb;
    substr($evalList, 5, 3) = $::FileExtArchiveFiles;
    ::log("creating evaluation list $evalList for testbed $tb\n");

    my $evalListFile = $::ListDir . $::sep . $evalList;
    my $anoRef = \%{$::lists{$tb}->{"packed"}->{"ano"}};
    makeEval($dirFile, $evalListFile, $anoRef) or do {
        ::log("ERROR: update of evaluation list ".$evalList.
              " failed !!!\n\n");
        next;
    };
    # create summary
    ::log("creating summary of $evalList for $tb\n");
    makeSum($evalListFile) or do {
        ::log("ERROR: update of summary ".$evalList.
              ".".$::FileExtSummary." failed !!!\n\n");
        next;
    };


    return 1;
}



# return a true value
1;