# Othercmd.pm
# implements all other AvtcTest commands
#
# v0.35, 2003-10-23
# Copyright (c) Michel Messerschmidt 2002,2003
#
package Avtc::Othercmd;
use strict;
use Exporter;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;

@Avtc::Othercmd::ISA = qw(Exporter);
@Avtc::Othercmd::EXPORT = qw(extract cleanTest cleanTestAll showConfig
                             showTestbeds showProducts showSystems showTests
                             showPacked showPackedCmd dumpData);

####################### implemented commands ###############################

sub extract {
    my $ret = 0;
    ::log("Working on test: $::configTestName\n");
    foreach my $os (sort (keys %::osList)) {
        LOOP_TB: foreach my $tb (sort keys %::lists) {
            my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
            my $LogDir = $::LogBase . $::sep . $os . $::sep . $tb;
            my $NewLogDir = $::NewLogBase . $::sep . $os . $::sep .
                            $::lists{$tb}->{"id"};
            my $TempDir = $::TempBase . $::sep . $os . $::sep . $tb;

            # check existance of TempDir and create it if nessecary
            ::makeDir($TempDir) or do {
                ::log("ERROR: creating directory $TempDir\n", 1);
                $ret = 1;
            };
            # change to TempDir, so all following IO can operate on current dir
            chdir($TempDir) or do {
                ::log("ERROR: chdir $TempDir failed:  $!\n", 1);
                ::log("Extraction of $tb data failed !\n\n", 4);
                $ret = 1;
                next LOOP_TB;
            };
            if ($::debug) {
                ::log("Current dir set to ". cwd() ."\n", 1);
            }
            LOOP_PROD: foreach my $prod (sort keys %::prodList) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 1);
                        }
                        next LOOP_PROD;
                    }
                }
                ::log("processing $prod - $tb - $os\n", 1);
                # define name of product related files
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $ful = $prod . "." . $::FileExtReport;
                my $archive = $EvalDir . $::sep . $prod . "." . $::ZipExt;
                my $status = ::getStatus($statFile);

                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod. ".
                          "Extraction skipped\n", 2);
                    $ret = 1;
                    next LOOP_PROD;
                } elsif ($status =~ /^NO/i or $status =~ /^-?S1( .*)?$/i) {
                    # there is no evaluation data for status -S1 and S1
                    # (and a status beginning with NO never has any results)
                    if ($::debug) {
                        ::log("status checked, skipping $prod\n", 2);
                    }
                    next LOOP_PROD;
                }
                # else: there are evaluation files for all other status codes
                #       that can be extracted
                if ($::configOs eq $::OS_WINDOWS) {
                    $archive = ::winPath($archive);
                }
                # The extraction program must extract all files per 
                # default (without specifying any filename) or this won't work
                # (or you need to add an wildcard to the end of the command)
                my $extCall = $::UnzipProg . " " . $::UnzipOpt . " " .
                   $archive;

                if ($::debug) {
                    ::log("calling $extCall\n", 2);
                }
                my $error = system($extCall);
                if ($::debug) {
                    ::log("...external program terminated with ".
                          "exitcode $error\n", 2);
                }
                if ($error) {
                    ::log("ERROR calling ".$::ZipProg.": exit code=".
                          $error.", error=$!\n", 2);
                    $ret = 1;
                }
            }  #end of foreach prod LOOP_PROD
        }  #end of foreach tb
    }  #end of foreach os
    return $ret;
}


sub cleanTestAll {
    return cleanTest("all");
}

# delete all files from a test except result data and some archives
# containing all relevant test evaluation files and logs
sub cleanTest {
    my $ok = 0;
    my $all = 0;
    if ($_[0] eq "all") {
        $all = 1;
    }

    # this is a dangerous function so print a warning first
    my $text = "*"x78 . "\n" .
             "This command will (recursively) delete all files in the " .
             "directories:\n  " . $::TempBase . "\n" .
             "  " . $::NewLogBase . "\n" .
             "and all files";
    if (not $all) {
        $text .= " except the archived logfiles (".$::LogArchiveName.".".
              $::ZipExt.")";
    }
    $text .= " in:\n"."  ".$::LogBase."\n" .
             "and all files except the result data (*.".$::FileExtResult.
             ") and the product status \nfiles (*".$::FileExtStatus.")";
    if (not $all) {
        $text .= " and the archived evaluation files (*.".$::ZipExt.")";
    }
    $text .= " in:\n"."  ".$::EvalBase."\n" .
             "\nAre you sure that you want to delete all other files ";
    if (not $::gui) {
        $text .= "(y/n) ?   ";
        ::log($text);
        my $tempAnswer = <STDIN>;
        chomp($tempAnswer);
        if (lc($tempAnswer) eq "y")  {
            $ok = 1;
        }
    } else {
        $text .= "?";
        my $dlg = $::mainWin->DialogBox(-title => "Confirm execution",
                                   -buttons => ["Ok", "Cancel"],
                                   -default_button => "Cancel");
        $dlg->add("Label", -anchor => 'nw', -justify => 'left',
                           -text => $text)
            ->pack();
        # show dialog
        my $button = $dlg->Show;
        if ($button eq "Ok") {
            $ok = 1;
        }
    }
    if (not $ok)  {
        ::log("...command cancelled.\n");
        return 1;
    }

    if ($all) {
        $text = "\n\nOnly the result data will remain. If any of these files".
                "\nis damaged, its contents will be lost forever and can't ".
                "be recovered!\n" .
                "You MUST have made a backup of all logfiles before you may ".
                "proceed.\n" .
                "\nHave you made a backup of all logfiles ";
        if (not $::gui) {
            $text .= "(y/n) ?   ";
            ::log($text);
            my $tempAnswer = <STDIN>;
            chomp($tempAnswer);
            if (lc($tempAnswer) ne "y")  {
                $ok = 0;
            }
        } else {
            $text .= "?";
            my $dlg = $::mainWin->DialogBox(-title => "Confirm execution",
                                       -buttons => ["Ok", "Cancel"],
                                       -default_button => "Cancel");
            $dlg->add("Label", -anchor => 'nw', -justify => 'left',
                               -text => $text)
                ->pack();
            # show dialog
            my $button = $dlg->Show;
            if ($button ne "Ok") {
                $ok = 0;
            }
        }
        if (not $ok)  {
            ::log("...command cancelled.\n");
            return 1;
        }
    } else {
        ::log("\n\nOnly redundant evaluation files will be deleted.\n".
              "But if any of the remaining files is damaged, all evaluation ".
              "steps\n must be done again!");
    }

    $text = "\nAny bug in this script function may cause severe data loss!\n".
            "\nAre you absolutely sure that you want to proceed ";
    if (not $::gui) {
        $text .= "(y/n) ?   ";
        ::log($text);
        my $tempAnswer = <STDIN>;
        chomp($tempAnswer);
        if (lc($tempAnswer) ne "y")  {
            $ok = 0;
        }
    } else {
        $text .= "?";
        my $dlg = $::mainWin->DialogBox(-title => "Confirm execution",
                                   -buttons => ["Ok", "Cancel"],
                                   -default_button => "Cancel");
        $dlg->add("Label", -anchor => 'nw', -justify => 'left',
                           -text => $text)
            ->pack();
        # show dialog
        my $button = $dlg->Show;
        if ($button ne "Ok") {
            $ok = 0;
        }
    }
    if (not $ok)  {
        ::log("...command cancelled.\n");
        return 1;
    }


    # now delete all files not listed in the warning above
    my $os;
    foreach $os (sort keys %::osList) {
        my $tb;
        LOOP_TB: foreach $tb (sort keys %::lists) {
            my @delFiles = ();
            my ($filename,$delCount);
            my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
            my $LogDir = $::LogBase . $::sep . $os . $::sep . $tb;
            my $NewLogDir = $::NewLogBase . $::sep . $os . $::sep .
                            $::lists{$tb}->{"id"};
            my $TempDir = $::TempBase . $::sep . $os . $::sep . $tb;
            if ($::debug) {
                ::log("   EvalDir = $EvalDir\n");
                ::log("    LogDir = $LogDir\n");
                ::log(" NewLogDir = $NewLogDir\n");
                ::log("   TempDir = $TempDir\n");
            }

            #create a list a of all files that should be deleted in this
            # os's and testbed's evaluation directory
            if (-e $EvalDir and -d $EvalDir) {
                ::log("    deleting files in $EvalDir\n");
                opendir(DIR, $EvalDir) or do {
                    ::log("  ERROR reading $EvalDir\n");
                    ::log("        !!! Can not delete files !!!\n");
                    next LOOP_TB;
                };
                while (defined($filename = readdir(DIR))) {
                    if (-d $filename or ($filename =~ /^\.\.?$/i)) {
                        # skip directory entries
                        next;
                    }
                    if (($filename =~ /$::FileExtStatus$/i) or
                        ($filename =~ /\.$::FileExtResult$/i)) {
                        if ($::debug) {
                            ::log("skipping ".$EvalDir.$::sep.$filename."\n",
                                  4);
                        }
                        next;
                    }
                    if (not $all and ($filename =~ /\.$::ZipExt$/i)) {
                        if ($::debug) {
                            ::log("skipping ".$EvalDir.$::sep.$filename."\n",
                                  4);
                        }
                        next;
                    }
                    push @delFiles, $EvalDir . $::sep . $filename;
                    if ($::debug) {
                        ::log("deleting ".$EvalDir.$::sep.$filename."\n", 2);
                    }
                }
                closedir(DIR);
                # delete files and check for errors
                $delCount = unlink @delFiles;
                if ($::debug) {
                    ::log("    $delCount files deleted\n");
                }
                if ($delCount < ($#delFiles + 1)) {
                    ::log("ERROR deleting files: ". $#delFiles+1-$delCount .
                          " files not deleted\n");
                }
            }

            #create a list a of all files that should be deleted in this
            # os's and testbed's logfile directory
            @delFiles = ();
            $filename = "";
            if (-e $LogDir and -d $LogDir) {
                ::log("    deleting files in $LogDir\n");
                opendir(DIR, $LogDir) or do {
                    ::log("  ERROR reading $LogDir\n");
                    ::log("        !!! Can not delete files !!!\n");
                    next LOOP_TB;
                };
                while (defined($filename = readdir(DIR))) {
                    if (-d $filename or ($filename =~ /^\.\.?$/i)) {
                        # skip directory entries
                        next;
                    }
                    if (not $all and ($filename =~ /^$::LogArchiveName\.$::ZipExt$/i)) {
                        if ($::debug) {
                            ::log("skipping ".$LogDir.$::sep.$filename."\n",
                                  4);
                        }
                        next;
                    }
                    push @delFiles, $LogDir . $::sep . $filename;
                    if ($::debug) {
                        ::log("    deleting ".$LogDir.$::sep.$filename."\n");
                    }
                }
                closedir(DIR);
                # delete files and check for errors
                $delCount = unlink @delFiles;
                if ($::debug) {
                    ::log("    $delCount files deleted\n");
                }
                if ($delCount < ($#delFiles + 1)) {
                    ::log("ERROR deleting files: ".. $#delFiles+1-$delCount .
                          " files not deleted\n");
                }
            }

            #create a list a of all files that should be deleted in this
            # os's and testbed's new logfile directory
            @delFiles = ();
            $filename = "";
            if (-e $NewLogDir and -d $NewLogDir) {
                ::log("    deleting files in $NewLogDir\n");
                opendir(DIR, $NewLogDir) or do {
                    ::log("  ERROR reading $NewLogDir\n");
                    ::log("        !!! Can not delete files !!!\n");
                    next LOOP_TB;
                };
                while (defined($filename = readdir(DIR))) {
                    if (-d $filename or ($filename =~ /^\.\.?$/i)) {
                        # skip directory entries
                        next;
                    }
                    # delete all files here
                    push @delFiles, $NewLogDir . $::sep . $filename;
                    if ($::debug) {
                        ::log("    deleting ".$NewLogDir.$::sep.$filename."\n");
                    }
                }
                closedir(DIR);
                # delete files and check for errors
                $delCount = unlink @delFiles;
                if ($::debug) {
                    ::log("    $delCount files deleted\n");
                }
                if ($delCount < ($#delFiles + 1)) {
                    ::log("ERROR deleting files: ". $#delFiles+1-$delCount .
                          " files not deleted\n");
                }
            }

            #create a list a of all files that should be deleted in this
            # os's and testbed's temporary directory
            @delFiles = ();
            $filename = "";
            if (-e $TempDir and -d $TempDir) {
                ::log("    deleting files in $TempDir\n");
                opendir(DIR, $TempDir) or do {
                    ::log("  ERROR reading $TempDir\n");
                    ::log("        !!! Can not delete files !!!\n");
                    next LOOP_TB;
                };
                while (defined($filename = readdir(DIR))) {
                    if (-d $filename or ($filename =~ /^\.\.?$/i)) {
                        # skip directory entries
                        next;
                    }
                    # delete all files here
                    push @delFiles, $TempDir . $::sep . $filename;
                    if ($::debug) {
                        ::log("    deleting ".$TempDir.$::sep.$filename."\n");
                    }
                }
                closedir(DIR);
                # ...and then delete files and check for errors
                $delCount = unlink @delFiles;
                if ($::debug) {
                    ::log("    $delCount files deleted\n");
                }
                if ($delCount < ($#delFiles + 1)) {
                    ::log("ERROR deleting files: ". $#delFiles+1-$delCount .
                          " files not deleted\n");
                }
            }
        }
    }
    return 0;
}


# show internal data structures
sub dumpData {
    ::log("Dump of all internal data structures\n".
          "------------------------------------\n");
    ::log("Debug level: $::debug\n", 1);
    ::log("using GUI: $::gui\n", 1);
    ::log("current status code for 'reset' command : $::resetStatus\n", 1);
    ::log("path separator for current system: $::sep\n", 1);
    ::log("path separator for scan reports: $::rps\n\n", 1);

    ::log("Known status codes: \n", 0);
    foreach my $s (sort keys %::statusCodes) {
        ::log("$s (". $::statusCodes{$s} .")\n", 1);
    }

    ::log("\nLists with selected objects:\n");
    ::log("\n".'Contents of HASH $::osList:'."\n");
    dumpDataRec(\%::osList, "  ");
    ::log("\n".'Contents of HASH $::prodList:'."\n");
    dumpDataRec(\%::prodList, "  ");
    ::log("\n".'Contents of $::lists:'."\n");
    dumpDataRec(\%::lists, "  ");
    ::log("\n".'Contents of HASH $::packList:'."\n");
    dumpDataRec(\%::packList, "  ");
    ::log("\n".'Contents of HASH $::noTestList:'."\n");
    dumpDataRec(\%::noTestList, "  ");

    ::log("\n\n\nAnd now the complete lists\n");
    ::log("\n".'Contents of HASH $::osListAll:'."\n");
    dumpDataRec(\%::osListAll, "  ");
    ::log("\n".'Contents of HASH $::prodListAll:'."\n");
    dumpDataRec(\%::prodListAll, "  ");
    ::log("\n".'Contents of $::listsAll:'."\n");
    dumpDataRec(\%::listsAll, "  ");
    ::log("\n".'Contents of HASH $::packListAll:'."\n");
    dumpDataRec(\%::packListAll, "  ");
    ::log("\n".'Contents of HASH $::noTestListAll:'."\n");
    dumpDataRec(\%::noTestListAll, "  ");
    return 0;
}

sub dumpDataRec {
    my $in = $_[0];
    my $head = $_[1];
    if (ref($in) eq "HASH") {
        my @klist = ();
        foreach my $k (sort keys %{$in}) {
            push @klist, $k;
            ::log($head."key:'".$k."'  -  value:'".${$in}{$k}."'\n");
            if (ref(${$in}{$k}) eq "HASH" or ref(${$in}{$k}) eq "ARRAY") {
                dumpDataRec(${$in}{$k}, $head."     ");
            }
        }
        if ($#klist == -1) {
            ::log($head."(empty hash)\n");
        }
    } elsif (ref($in) eq "ARRAY") {
        foreach my $i (sort @{$in}) {
            dumpDataRec($i, $head."     ");
        }
        if ($#{$in} == -1) {
            ::log($head."(empty array)\n");
        }
    } else {
        ::log($head."'".$in."'\n");
    }
}


# display data from configuration file
sub showTestbeds {
    my $c = 0;
    if (not $::quiet) {
        ::log("List of all testbeds:\n", 0);
        ::log("Type      ID  Name,Path,Lists\n". "-"x50 ."\n", 1);
    }
    foreach my $i (sort keys %::listsAll) {
        if (not $::quiet) {
            ::log($i."  ".$::listsAll{$i}->{"id"}."   Name: ".
                  $::listsAll{$i}->{"name"}."\n", 1);
            ::log("Win-Path: ".$::listsAll{$i}->{"path"}->{$::OS_WINDOWS}.
                  "   Unix-Path: ".$::listsAll{$i}->{"path"}->{$::OS_UNIX}."\n", 8);
            if (substr($i, 3, 1) ne "_") {
                ::log("is packed", 8);
                if (not exists ($::listsAll{$i}->{"packed"})) {
                    ::log(" but packed data is MISSING !!!\n");
                } else {
                    ::log("\n");
                }
                if ($::debug and exists ($::listsAll{$i}->{"packed"})) {
                    ::log("found '{packed}'\n", 8);
                }
                if ($::debug and exists ($::listsAll{$i}->{"packed"}->{"archlist"})) {
                    ::log("found 'packed->archlist'\n", 8);
                }

                ::log("Evaluation lists: \n", 8);
                ::log($::listsAll{$i}->{"packed"}->{"archlist"}.
                      " (archive evaluation)\n", 8);
                ::log($i." (archive contents evaluation)\n", 8);
            } else {
                ::log("is not packed, ", 8);
                if (not exists ($::listsAll{$i}->{"single"})) {
                    ::log("has multiple evallists\n", 0);
                    ::log("Evaluation lists: \n", 8);
                    foreach my $j (sort keys %{$::listsAll{$i}->{"evals"}}) {
                        ::log($j."   Eval-ID = '".
                              $::listsAll{$i}->{"evals"}->{$j}->{"id"}."'     Name= '".
                              $::listsAll{$i}->{"evals"}->{$j}->{"name"}."'\n", 8);
                    }
                } else {
                    ::log("is single\n", 0);
                    ::log("Evaluation lists: \n", 8);
                    foreach my $j (sort keys %{$::listsAll{$i}->{"evals"}}) {
                        ::log($j."   Eval-ID = '".
                              $::listsAll{$i}->{"evals"}->{$j}->{"id"}."'\n",
                              8);
                    }
                }
                if ($::debug and exists ($::listsAll{$i}->{"packed"})) {
                    ::log("ERROR: found 'packed->archlist'\n", 8);
                }
                if ($::debug and exists ($::listsAll{$i}->{"packed"}->{"archlist"})) {
                    ::log("ERROR: found 'packed->archlist'\n", 8);
                }
            }
        } else {
            ::log($i." ".$::listsAll{$i}->{"name"}, 0);
        }
        ::log("\n", 0);
        $c++;
    }
    if (not $::quiet) {
        ::log("\n");
        ::log("Total testbeds: ".$c."\n", 1);
    }
    return 0;
}


# display data from configuration file
sub showProducts {
    my $c = 0;
    if (not $::quiet) {
        ::log("List of all products: \n", 0);
        ::log("Code  Name\n". "-"x29 ."\n", 1);
    }
    foreach my $i (sort keys %::prodListAll) {
        ::log($i."   ".$::prodListAll{$i}."\n", 1);
        $c++;
    }
    if (not $::quiet) {
        ::log("\n");
        ::log("Total products: ".$c."\n", 1);
    }
    return 0;
}


# display data from configuration file
sub showSystems {
    my $c = 0;
    if (not $::quiet) {
        ::log("List of all operating systems: \n", 0);
        ::log("Code  Name\n". "-"x29 ."\n", 1);
    }
    foreach my $i (sort keys %::osListAll) {
        ::log($i."   ".$::osListAll{$i}."\n", 1);
        $c++;
    }
    if (not $::quiet) {
        ::log("\n");
        ::log("Total systems: ".$c."\n", 1);
    }
    return 0;
}


# display data from global configuration file
sub showTests {
    if (not $::quiet) {
        ::log("List of all operating systems: \n", 0);
        ::log("Testname     Configuration-File\n" . "-"x70 . "\n", 1);
    }
    foreach my $i (sort keys %::configAllTests) {
        if (not $::quiet) {
            my $fill = 13-length($i);
            ::log($i . " "x$fill . $::configAllTests{$i} . "\n", 1);
        } else {
            ::log($i."\n", 0);
        }
    }
    return 0;
}


# display data from configuration file
sub showPacked {
    my $c = 0;
    my %tb = ();
    foreach my $i (sort keys %::listsAll) {
        $tb{$i} = 0;
    }
    if (not $::quiet) {
        ::log("List of all compression programs (and their use in the ".
              "testbeds):\n", 0);
        ::log("Code  Testbeds  Name\n". "-"x30 ."\n", 1);
    }
    foreach my $i (sort keys %::packListAll) {
        my $tbnr = 0;
        foreach my $j (sort keys %::listsAll) {
            if (exists($::listsAll{$j}->{"packed"}->{"prg"}->{$i})) {
                $tbnr++;
                $tb{$j}++;
            }
        }
        if ($tbnr) {
            if (not $::quiet) {
                ::log(sprintf("%3s     %2d      %s\n", $i, $tbnr,
                              $::packListAll{$i}->{"name"}),
                      1);
            } else {
                ::log($i."  ".$::packListAll{$i}->{"name"}."\n", 0);
            }
            $c++;
        }
    }
    if (not $::quiet) {
        ::log("\n", 0);
        foreach my $i (sort keys %::listsAll) {
            if ($tb{$i}) {
                ::log("Total in testbed ".$i.": ".$tb{$i}."\n", 1);
            }
        }
        ::log("Total number of compression programs: ".$c."\n", 1);
    }
    return 0;
}


# display data from configuration file
sub showPackedCmd {
    my $c = 0;
    ::log("Compression Programs: \n", 0);
    foreach my $i (sort keys %::packListAll) {
        ::log(sprintf("Code: %3s    Name: %s\n", $i,
                      $::packListAll{$i}->{"name"}),
              1);
        foreach my $j (sort keys %::listsAll) {
            if (exists($::listsAll{$j}->{"packed"}->{"prg"}->{$i})) {
                ::log($j.":   Add command: '".
                      $::listsAll{$j}->{"packed"}->{"prg"}->{$i}->{"add"}.
                      "'\n", 2);
                ::log("Only Relative Paths: ", 8);
                if ($::listsAll{$j}->{"packed"}->{"prg"}->{$i}->{"onlyRelative"}) {
                    ::log("yes");
                } else {
                    ::log("no");
                }
                ::log("      Filename = '".
                      $::listsAll{$j}{"packed"}{"prg"}{$i}{"filename"}."'\n");
                ::log("Only Compression: ", 8);
                if ($::listsAll{$j}->{"packed"}->{"prg"}->{$i}->{"onlyComp"}) {
                    ::log("yes");
                } else {
                    ::log("no");
                }
                ::log("         Only Archive: ");
                if ($::listsAll{$j}->{"packed"}->{"prg"}->{$i}->{"onlyArchive"}) {
                    ::log("yes\n");
                } else {
                    ::log("no\n");
                }
            }
        }
        $c++;
    }
    ::log("\n");
    ::log("Total number of compression programs: ".$c."\n", 1);
    return 0;
}


# display data from configuration file and check for undefined variables
sub showConfig {
    ::log("Running on a $::configOs system\n", 1);
    if ($::configGlobalFile) {
        ::log("    global config = $::configGlobalFile\n");
    } else {
        ::log("ERROR: configGlobalFile not defined !!!\n");
    }
    if ($::configTestName) {
        ::log("  unique testname = $::configTestName\n");
    } else {
        ::log("ERROR: configTestName not defined !!!\n");
    }
    if ($::TestName) {
        ::log("   Final Testname = $::TestName\n");
    } else {
        ::log("ERROR: TestName not defined !!!\n");
    }
    if ($::TestType) {
        ::log("         TestType = $::TestType\n");
        if ($::TestType eq "heureka") {
            if ($::RefTestName) {
            ::log("      RefTestname = $::RefTestName\n");
            } else {
            ::log("ERROR: RefTestName not defined !!!\n");
            }
            if ($::TestMonth3Name) {
            ::log("          3MonExt = $::TestMonth3Name\n");
            } else {
            ::log("ERROR: TestMonth3Name not defined !!!\n");
            }
            if ($::TestMonth6Name) {
            ::log("          6MonExt = $::TestMonth6Name\n");
            } else {
            ::log("ERROR: TestMonth6Name not defined !!!\n");
            }
        }
    } else {
        ::log("ERROR: TestType not defined !!!\n");
    }
    ::log("Test files use $::configSyntax syntax\n", 1);
    if ($::configTestFile) {
        ::log("      config file = $::configTestFile\n");
    } else {
        ::log("ERROR: configTestFile not defined !!!\n");
    }
    if ($::DirListDir) {
        ::log("       DirListDir = $::DirListDir\n");
    } else {
        ::log("ERROR: DirListDir not defined !!!\n");
    }
    if ($::TempRoot) {
        ::log("         TempRoot = $::TempRoot\n");
    } else {
        ::log("ERROR: TempRoot not defined !!!\n");
    }
    if ($::EvalRoot) {
        ::log("         EvalRoot = $::EvalRoot\n");
    } else {
        ::log("ERROR: EvalRoot not defined !!!\n");
    }
    if ($::LogRoot) {
        ::log("          LogRoot = $::LogRoot\n");
    } else {
        ::log("ERROR: LogRoot not defined !!!\n");
    }
    if ($::NewLogRoot) {
        ::log("       NewLogRoot = $::NewLogRoot\n");
    } else {
        ::log("ERROR: NewLogRoot not defined !!!\n");
    }
    if ($::InfoRoot) {
        ::log("         InfoRoot = $::InfoRoot\n");
    } else {
        ::log("ERROR: InfoRoot not defined !!!\n");
    }
    if ($::DocRoot) {
        ::log("          DocRoot = $::DocRoot\n");
    } else {
        ::log("ERROR: DocRoot not defined !!!\n");
    }
    if ($::TempBase) {
        ::log("         TempBase = $::TempBase\n");
    } else {
        ::log("ERROR: TempBase not defined !!!\n");
    }
    if ($::EvalBase) {
        ::log("         EvalBase = $::EvalBase\n");
    } else {
        ::log("ERROR: EvalBase not defined !!!\n");
    }
    if ($::LogBase) {
        ::log("          LogBase = $::LogBase\n");
    } else {
        ::log("ERROR: LogBase not defined !!!\n");
    }
    if ($::NewLogBase) {
        ::log("       NewLogBase = $::NewLogBase\n");
    } else {
        ::log("ERROR: NewLogBase not defined !!!\n");
    }
    if ($::ResultDir) {
        ::log("        ResultDir = $::ResultDir\n");
    } else {
        ::log("ERROR: ResultDir not defined !!!\n");
    }
    if ($::ReportDir) {
        ::log("        ReportDir = $::ReportDir\n");
    } else {
        ::log("ERROR: ReportDir not defined !!!\n");
    }
    if ($::KeyDir) {
        ::log("           KeyDir = $::KeyDir\n");
    } else {
        ::log("ERROR: KeyDir not defined !!!\n");
    }
    if ($::ListDir) {
        ::log("          ListDir = $::ListDir\n");
    } else {
        ::log("ERROR: ListDir not defined !!!\n");
    }
    if ($::TestStatus) {
        ::log(" Test Status file = $::TestStatus\n");
    } else {
        ::log("ERROR: TestStatus not defined !!!\n");
    }
    if ($::TestStatusEval) {
        ::log(" Eval Status file = $::TestStatusEval\n");
    } else {
        ::log("ERROR: TestStatusEval not defined !!!\n");
    }
    if ($::TestOs) {
        ::log("           OSList = $::TestOs\n");
    } else {
        ::log("ERROR: TestOs not defined !!!\n");
    }
    if ($::TestTb) {
        ::log("      TestbedList = $::TestTb\n");
    } else {
        ::log("ERROR: TestTb not defined !!!\n");
    }
    if ($::TestProd) {
        ::log("      ProductList = $::TestProd\n");
    } else {
        ::log("ERROR: TestProd not defined !!!\n");
    }
    if ($::TestNoTest) {
        ::log("       NoTestList = $::TestNoTest\n");
    } else {
        ::log("ERROR: TestNoTest not defined !!!\n");
    }
    if ($::TestPacked) {
        ::log("   PackedProgList = $::TestPacked\n");
    } else {
        ::log("ERROR: TestPacked not defined !!!\n");
    }
    if ($::extSort) {
        if ($::SortProg) {
            ::log("             Sort = $::SortProg\n");
        } else {
            ::log("ERROR: SortProg not defined !!!\n");
        }
    }
    if ($::ZipProg) {
        ::log("      Zip program = $::ZipProg\n");
    } else {
        ::log("ERROR: ZipProg not defined !!!\n");
    }
    if ($::UnzipProg) {
        ::log("    Unzip program = $::UnzipProg\n");
    } else {
        ::log("ERROR: UnzipProg not defined !!!\n");
    }
    if ($::LogArchiveName) {
        ::log("   LogArchiveName = $::LogArchiveName\n");
    } else {
        ::log("ERROR: LogArchiveName not defined !!!\n");
    }
    if ($::OldLogArchiveName) {
        ::log("OldLogArchiveName = $::OldLogArchiveName\n");
    } else {
        ::log("ERROR: OldLogArchiveName not defined !!!\n");
    }
    if ($::ZipExt) {
        ::log("   file extension = $::ZipExt\n");
    } else {
        ::log("ERROR: ZipExt not defined !!!\n");
    }
    if ($::ZipOpt) {
        ::log("      add options = '$::ZipOpt'\n");
    } else {
        ::log("ERROR: ZipOpt not defined !!!\n");
    }
    if ($::UnzipOpt) {
        ::log("  extract options = '$::UnzipOpt'\n");
    } else {
        ::log("ERROR: UnzipOpt not defined !!!\n");
    }
    if ($::ZipDelOpt) {
        ::log("   delete options = '$::ZipDelOpt'\n");
    } else {
        ::log("ERROR: ZipDelOpt not defined !!!\n");
    }

    # and some internal tests
    if (not defined $::EOL) {
        ::log("ERROR: EOL not defined !!!\n");
    }
    if (not defined $::VirusNameUI) {
        ::log("ERROR: VirusNameUI not defined !!!\n");
    }
    if (not defined $::FileExtText) {
        ::log("ERROR: FileExtText not defined !!!\n");
    }
    if (not defined $::FileExtLists) {
        ::log("ERROR: FileExtLists not defined !!!\n");
    }
    if (not defined $::FileExtVariants) {
        ::log("ERROR: FileExtVariants not defined !!!\n");
    }
    if (not defined $::FileExtSamples) {
        ::log("ERROR: FileExtSamples not defined !!!\n");
    }
    if (not defined $::FileExtSummary) {
        ::log("ERROR: FileExtSummary not defined !!!\n");
    }
    if (not defined $::FileExtFP) {
        ::log("ERROR: FileExtFP not defined !!!\n");
    }
    if (not defined $::FileExtAnomaly) {
        ::log("ERROR: FileExtAnomaly not defined !!!\n");
    }
    if (not defined $::FileExtExcluded) {
        ::log("ERROR: FileExtExcluded not defined !!!\n");
    }
    if (not defined $::FilePreDirlist) {
        ::log("ERROR: FilePreDirlist not defined !!!\n");
    }
    if (not defined $::FileExtReport) {
        ::log("ERROR: FileExtReport not defined !!!\n");
    }
    if (not defined $::FileExtNormUnsorted) {
        ::log("ERROR: FileExtNormUnsorted not defined !!!\n");
    }
    if (not defined $::FileExtScannedOK) {
        ::log("ERROR: FileExtScannedOK not defined !!!\n");
    }
    if (not defined $::FileExtAuswert) {
        ::log("ERROR: FileExtAuswert not defined !!!\n");
    }
    if (not defined $::FileExtAuswertRedundant) {
        ::log("ERROR: FileExtAuswertRedundant not defined !!!\n");
    }
    if (not defined $::FileExtReportRem) {
        ::log("ERROR: FileExtReportRem not defined !!!\n");
    }
    if (not defined $::FileExtControl) {
        ::log("ERROR: FileExtControl not defined !!!\n");
    }
    if (not defined $::FileExtCutList) {
        ::log("ERROR: FileExtCutList not defined !!!\n");
    }
    if (not defined $::FileExtCorrupt) {
        ::log("ERROR: FileExtCorrupt not defined !!!\n");
    }
    if (not defined $::FileReportProblems) {
        ::log("ERROR: FileReportProblems not defined !!!\n");
    }
    if (not defined $::FileReportProdInfo) {
        ::log("ERROR: FileReportProdInfo not defined !!!\n");
    }
    if (not defined $::FileCmpAll) {
        ::log("ERROR: FileCmpAll not defined !!!\n");
    }
    if (not defined $::FileExtStatus) {
        ::log("ERROR: FileExtStatus not defined !!!\n");
    }
    if (not defined $::FileExtCmpF) {
        ::log("ERROR: FileExtCmpF not defined !!!\n");
    }
    if (not defined $::FileExtCmpV) {
        ::log("ERROR: FileExtCmpV not defined !!!\n");
    }
    if (not defined $::FileExtCmpWinF) {
        ::log("ERROR: FileExtCmpWinF not defined !!!\n");
    }
    if (not defined $::FileExtCmpWinV) {
        ::log("ERROR: FileExtCmpWinV not defined !!!\n");
    }
    if (not defined $::FileExtScan1) {
        ::log("ERROR: FileExtScan1 not defined !!!\n");
    }
    if (not defined $::FileExtScan2) {
        ::log("ERROR: FileExtScan2 not defined !!!\n");
    }
    if (not defined $::FileExtScan3) {
        ::log("ERROR: FileExtScan3 not defined !!!\n");
    }
    if (not defined $::FileExtPSL) {
        ::log("ERROR: FileExtPSD not defined !!!\n");
    }
    if (not defined $::FileExtNorm) {
        ::log("ERROR: FileExtNorm not defined !!!\n");
    }
    if (not defined $::FileExtNormRemPacked) {
        ::log("ERROR: FileExtNormRemPacked not defined !!!\n");
    }
    if (not defined $::FileExtScanOKRemPacked) {
        ::log("ERROR: FileExtScanOKRemPacked not defined !!!\n");
    }
    if (not defined $::FileExtPackedAdd) {
        ::log("ERROR: FileExtPackedAdd not defined !!!\n");
    }
    if (not defined $::FileExtPackedOKAdd) {
        ::log("ERROR: FileExtPackedOKAdd not defined !!!\n");
    }
    if (not defined $::FileExtDetFiles) {
        ::log("ERROR: FileExtDetFiles not defined !!!\n");
    }
    if (not defined $::FileExtMissedFiles) {
        ::log("ERROR: FileExtMissedFiles not defined !!!\n");
    }
    if (not defined $::FileExtDetFPFiles) {
        ::log("ERROR: FileExtDetFPFiles not defined !!!\n");
    }
    if (not defined $::FileExtMissedFPFiles) {
        ::log("ERROR: FileExtMissedFPFiles not defined !!!\n");
    }
    if (not defined $::FileExtAnoFiles) {
        ::log("ERROR: FileExtAnoFiles not defined !!!\n");
    }
    if (not defined $::FileExtGarbageFiles) {
        ::log("ERROR: FileExtGarbageFiles not defined !!!\n");
    }
    if (not defined $::FileExtExcludedFiles) {
        ::log("ERROR: FileExtExcludedFiles not defined !!!\n");
    }
    if (not defined $::FileExtScanFiles) {
        ::log("ERROR: FileExtScanFiles not defined !!!\n");
    }
    if (not defined $::FileExtOKFiles) {
        ::log("ERROR: FileExtOKFiles not defined !!!\n");
    }
    if (not defined $::FileExtInfAndOK) {
        ::log("ERROR: FileExtInfAndOK not defined !!!\n");
    }
    if (not defined $::FileExtPostscan) {
        ::log("ERROR: FileExtPostscan not defined !!!\n");
    }
    if (not defined $::FileExtGarbage) {
        ::log("ERROR: FileExtGarbage not defined !!!\n");
    }
    if (not defined $::FileExtEval) {
        ::log("ERROR: FileExtEval not defined !!!\n");
    }
    if (not defined $::FileExtRem) {
        ::log("ERROR: FileExtRem not defined !!!\n");
    }
    if (not defined $::FileExtDetVir) {
        ::log("ERROR: FileExtDetVir not defined !!!\n");
    }
    if (not defined $::FileExtUnrId) {
        ::log("ERROR: FileExtUnrId not defined !!!\n");
    }
    if (not defined $::FileExtMultInf) {
        ::log("ERROR: FileExtMultInf not defined !!!\n");
    }
    if (not defined $::FileExtMultScan) {
        ::log("ERROR: FileExtMultScan not defined !!!\n");
    }
    if (not defined $::FileExtFilteredAno) {
        ::log("ERROR: FileExtFilteredAno not defined !!!\n");
    }
    if (not defined $::FileExtResult) {
        ::log("ERROR: FileExtResult not defined !!!\n");
    }
    if (not defined $::FileExtArchiveFiles) {
        ::log("ERROR: FileExtArchiveFiles not defined !!!\n");
    }
    if (not defined $::FileExtPackedList) {
        ::log("ERROR: FileExtPackedList not defined !!!\n");
    }
    if (not defined $::FileIdFP) {
        ::log("ERROR: FileIdFP not defined !!!\n");
    }
    return 0;
}


# return a true value
1;