# Report.pm
# implements 'result' and 'status' commands
#
# v0.64, 2004-03-29
# Copyright (c) Michel Messerschmidt 2002-2004
#
package Avtc::Report;
use strict;
use Exporter;
use File::Copy;

@Avtc::Report::ISA = qw(Exporter);
@Avtc::Report::EXPORT = qw(reportResult exportResult);

sub exportResult {
    my $ret1 = resultTextCSV();
    my $ret2 = ::reportStatus();
    return $ret1 + $ret2;
}


sub reportResult {
    my $ret1 = resultTextTables();
    my $ret2 = ::reportStatus();
    return $ret1 + $ret2;
#    my $ret2 = resultTextCSV();
#    my $ret3 = ::reportStatus();
#    return $ret1 + $ret2 + $ret3;
}


# create Comma-Separated-Value result tables
sub resultTextCSV {
    my $ret = 0;
    # get all result data
    getData();

    # create special result tables for packed testbeds
    foreach my $os (sort keys %::osList) {
        LOOP_TB: foreach my $tb (sort keys %::lists) {
            # check for not tested combinations
            foreach my $skip (@::noTestListAll) {
                if ($skip eq "$os $tb ALL") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - ALL\n", 2);
                    }
                    next LOOP_TB;
                }
            }
            if (exists $::lists{$tb}->{"packed"}) {
                my $refTb = $tb;
                substr($refTb, 3, 2) = "__";

                $ret += createPackedCSVLists($os, $tb);
                $ret += createPackedLossCSVLists($os, $tb, $refTb);
            } else {
                $ret = createCSVLists($os, $tb);
            }
        }
    }
    # delete results from dynamic data hash (as it is reused)
    delete($::data{"result"});
    delete($::data{"resultLine"});

    return $ret;
}


# create old style ASCII result tables
sub resultTextTables {
    my $ret = 0;
    # get all result data
    getData();

    # create special result tables for packed testbeds
    foreach my $os (sort keys %::osList) {
        LOOP_TB: foreach my $tb (sort keys %::lists) {
            # check for not tested combinations
            foreach my $skip (@::noTestListAll) {
                if ($skip eq "$os $tb ALL") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - ALL\n", 2);
                    }
                    next LOOP_TB;
                }
            }
            if (exists $::lists{$tb}->{"packed"}) {
                my $refTb = $tb;
                substr($refTb, 3, 2) = "__";

                $ret += processPackedLists($os, $tb);
                $ret += processPackedComp($os, $tb);
                $ret += createPackedArchResults($os, $tb);
                $ret += createPackedLossResults($os, $tb, $refTb);
            } else {
                $ret = processTestbedLists($os, $tb);
            }
        }
    }
    # delete results from dynamic data hash (as it is reused)
    delete($::data{"result"});
    delete($::data{"resultLine"});

    return $ret;
}


sub processTestbedLists {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    LOOP_EVAL: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        ::log("creating result table for: $os - $list\n", 1);
        my $resultFile = $::ResultDir . $::sep . $list . "." . $os;

        open(RESULT, "> $resultFile") or do {
            ::log("ERROR creating $resultFile: $!\n", 1);
            ::log("!!! Result generation of $list stopped !!!\n\n", 4);
            $ret = 1;
            next LOOP_EVAL;
        };
        binmode(RESULT);

        printf(RESULT 'Table %3s.%1s : Results of "%s" under %s%s',
               $os, substr($tb,0,1),
               $::listsAll{$tb}->{"evals"}->{$list}->{"name"},
               $::osListAll{$os}, $::EOL);
        print(RESULT "===============================================".
              "=============", $::EOL);
        if ($id eq "F") {
            # write result table header
            print(RESULT "Scanner     False positives detected\n");
            print(RESULT "-------------------------------------------".
                  "-----------------", $::EOL);
            printf(RESULT "Testbed  %6d 100.0%%                      ".
                   "      %s",
                   $::lists{$tb}->{"evals"}->{$list}->{"sumF"}, $::EOL);
            print(RESULT "-------------------------------------------".
                  "-----------------", $::EOL);
        } else {
            # write result table header
            print(RESULT "                           This  includes", $::EOL);
            print(RESULT "            Viruses     ---- unreliably ----     ".
                  "Files", $::EOL);
            print(RESULT "Scanner     detected    identified   detected   ".
                  "detected", $::EOL);
            print(RESULT "-------------------------------------------".
                  "-----------------", $::EOL);
            printf(RESULT "Testbed  %6d 100.0%%                        %6d ".
                   "100.0%%%s", $::lists{$tb}->{"evals"}->{$list}->{"sumV"},
                   $::lists{$tb}->{"evals"}->{$list}->{"sumF"}, $::EOL);
            print(RESULT "-------------------------------------------".
                  "-----------------", $::EOL);
        }
        LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
            # check for not tested combinations
            foreach (@::noTestListAll) {
                if ($_ eq "$os $tb $prod") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - $prod\n", 2);
                    }
                    next LOOP_PROD;
                }
            }
            # define name of product related files
            my $prodEval = $prod;
            if ($id ne "") {
                $prodEval .= "_" . $id;
            }
            my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
            my $data = $::data{"resultLine"}->{$os}->{$tb}->{$prodEval};
            $ret += processProduct($prod, $statFile, $data);
        }
        close(RESULT);
    }

    return $ret;
}


sub processPackedLists {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    # list of all compression programs used in this testbed
    my %packPrgs = %{$::lists{$tb}->{"packed"}->{"prg"}};
    foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        LOOP_PACK: foreach my $pack (sort keys %packPrgs) {
            ::log("creating result table for: $os - $list - $pack\n", 1);
            my $tempList = $list;
            substr($tempList, 5, 3) = $pack;
            my $resultFile = $::ResultDir . $::sep . $tempList . "." . $os;

            open(RESULT, "> $resultFile") or do {
                ::log("ERROR creating $resultFile: $!\n", 1);
                ::log("!!! Result generation of $list stopped !!!\n\n", 4);
                $ret = 1;
                next LOOP_PACK;
            };
            binmode(RESULT);

            # write result table header
            my $header = sprintf('Table %3s.%1s : Results of "%s" under %s '.
                         'packed with %s', $os, substr($tb, 0, 1),
                         $::lists{$tb}->{"evals"}->{$list}->{"name"},
                         $::osList{$os}, $packPrgs{$pack}->{"name"});
            $header = ::indentString($header, ": ", 78);
            print(RESULT $header, $::EOL, "="x78, $::EOL);

            if ($id eq "F") {
                # write result table header
                print(RESULT "Scanner     False positives detected", $::EOL);
                print(RESULT "---------------------------------------".
                      "---------------------", $::EOL);
                printf(RESULT "Testbed  %6d 100.0%%                  ".
                       "          %s",
                       $::lists{$tb}->{"evals"}->{$list}->{"sumF"}, $::EOL);
                print(RESULT "---------------------------------------".
                      "---------------------", $::EOL);
            } else {
                # write result table header
                print(RESULT "                           This  includes",
                      $::EOL);
                print(RESULT "            Viruses     ---- unreliably ----".
                      "     Files", $::EOL);
                print(RESULT "Scanner     detected    identified   detected".
                      "   detected", $::EOL);
                print(RESULT "---------------------------------------".
                      "---------------------", $::EOL);
                printf(RESULT "Testbed  %6d 100.0%%                        ".
                       "%6d 100.0%%%s",
                       $::lists{$tb}->{"evals"}->{$list}->{"sumV"},
                       $::lists{$tb}->{"evals"}->{$list}->{"sumF"}, $::EOL);
                print(RESULT "---------------------------------------".
                      "---------------------", $::EOL);
            }
            LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 2);
                        }
                        next LOOP_PROD;
                    }
                }
                # define name of product related files
                my $prodEval = $prod . $pack;
                if ($id ne "") {
                    $prodEval .= "_" . $id;
                }
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $data = $::data{"resultLine"}->{$os}->{$tb}->{$prodEval};
                $ret += processProduct($prod, $statFile, $data);
            }
            close(RESULT);
        }   #end of foreach LOOP_EVAL
    }

    return $ret;
}


sub processProduct {
    my $ret = 0;
    my $prod = $_[0];
    my $statFile = $_[1];
    my $data = $_[2];
    # get status of current product
    my $status = ::getStatus($statFile);

    if ($status eq "") {
        ::log("ERROR getting status for $prod\n", 2);
        print(RESULT $prod . "*   *** ERROR ***" . $::EOL);
        $ret = 1;
    } elsif ($status =~ /^NO TEST( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } elsif ($status =~ /^NO SUPPORT( .*)?$/i) {
        printf(RESULT "%s    *** testbed not supported by this product ".
               "    ***%s", $prod, $::EOL);
    } elsif ($status =~ /^NO REPORT( .*)?$/i) {
        printf(RESULT "%s    *** no report - see problem list          ".
               "    ***%s", $prod, $::EOL);
    } elsif ($status =~ /^-S1( .*)?$/i) {
        printf(RESULT "%s*   ** no report yet                          ".
               "     **%s", $prod, $::EOL);
    } elsif ($status =~ /^S1( .*)?$/i) {
        printf(RESULT "%s*   ** no evaluation yet                      ".
               "     **%s", $prod, $::EOL);
    } else {
        if ($status =~ /^S1E/i) {
            substr($data, 3, 1) = "*";
        } elsif ($status =~ /^-?S(\d)E?/i) {
            my $postscanNr = $1 - 1;
            substr($data, 3, 3) = "*p" . $postscanNr;
        }
        print(RESULT $data, $::EOL);
    }

    return $ret;
}


sub processPackedComp {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    # list of all compression programs used in this testbed
    my %packPrgs = %{$::lists{$tb}->{"packed"}->{"prg"}};
    LOOP_PACKCMP: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        ::log("creating packed 'compare' result tables for: $os - $list\n", 1);
        my $tempList = $list;
        substr($tempList, 7, 1) = "V";
        my $resultFileV = $::ResultDir . $::sep . $tempList . "." . $os;

        substr($tempList, 7, 1) = "F";
        my $resultFileF = $::ResultDir . $::sep . $tempList . "." . $os;

        open(RESULTV, "> $resultFileV") or do {
            ::log("ERROR creating $resultFileV: $!\n", 1);
            ::log("!!! Result generation of $list stopped !!!\n\n", 4);
            $ret = 1;
            next LOOP_PACKCMP;
        };
        binmode(RESULTV);
        open(RESULTF, "> $resultFileF") or do {
            ::log("ERROR creating $resultFileF: $!\n", 1);
            ::log("!!! Result generation of $list stopped !!!\n\n", 4);
            $ret = 1;
            next LOOP_PACKCMP;
        };
        binmode(RESULTF);

        # write result table header
        my $header = sprintf('Table %3s.%1s : Results of "%s" under %s',
                     $os, substr($tb, 0, 1),
                     $::lists{$tb}->{"evals"}->{$list}->{"name"},
                     $::osList{$os});
        $header = ::indentString($header, ": ", 78);
        print(RESULTV $header, $::EOL, "="x78, $::EOL);
        print(RESULTF $header, $::EOL, "="x78, $::EOL);
        print(RESULTV "        Viruses detected per compression program");
        print(RESULTF "        Files detected per compression program");

        # Divide all compression programs into sections, each containing
        # 5 programs. These are used to create better formatted tables
        my $sec = 0;
        my $nr = 0;
        my %packSec = ();
        foreach my $pack (sort keys %packPrgs) {
            push @{$packSec{$sec}}, $pack;
            $nr++;
            if (($nr % 5) == 0) {
                $sec++;
            }
        }

        foreach my $sec (sort keys %packSec) {
            my @packList = @{$packSec{$sec}};
            print(RESULTV $::EOL, "-------+");
            print(RESULTF $::EOL, "-------+");
            foreach my $pack (@packList) {
                print(RESULTV "-------------+");
                print(RESULTF "-------------+");
            }
            print(RESULTV $::EOL, "       |");
            print(RESULTF $::EOL, "       |");
            foreach my $pack (@packList) {
				my $packname = $::packListAll{$pack}->{"name"};
				if (length($packname) > 9) {
                    $packname = substr($pack, 0, 9);
                }
                printf(RESULTV "  %-9s  |", $packname);
                printf(RESULTF "  %-9s  |", $packname);
            }
            print(RESULTV $::EOL, "-------+");
            print(RESULTF $::EOL, "-------+");
            foreach my $pack (@packList) {
                print(RESULTV "-------------+");
                print(RESULTF "-------------+");
            }

            print(RESULTV $::EOL, "Testbed");
            print(RESULTF $::EOL, "Testbed");
            foreach my $pack (@packList) {
                printf(RESULTV "| %4d 100.0%% ",
                       $::lists{$tb}{"evals"}{$list}{"sumV"});
                printf(RESULTF "| %4d 100.0%% ",
                       $::lists{$tb}{"evals"}{$list}{"sumF"});
            }
            print(RESULTV "|", $::EOL, "-------+");
            print(RESULTF "|", $::EOL, "-------+");
            foreach my $pack (@packList) {
                print(RESULTV "-------------+");
                print(RESULTF "-------------+");
            }
            print(RESULTV $::EOL);
            print(RESULTF $::EOL);
            LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 2);
                        }
                        next LOOP_PROD;
                    }
                }
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                # get status of current product
                my $status = ::getStatus($statFile);
                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod\n", 2);
                    $ret = 1;
                    print(RESULTV $prod . "*   | *** ERROR ***" . $::EOL);
                    print(RESULTF $prod . "*   | *** ERROR ***" . $::EOL);
                } elsif ($status =~ /^NO TEST( .*)?$/i) {
                    # it makes no sense here to include the product in the table
                } elsif ($status =~ /^NO SUPPORT( .*)?$/i) {
                    printf(RESULTV "%s    | *** testbed not supported by this product     ***%s", $prod, $::EOL);
                    printf(RESULTF "%s    | *** testbed not supported by this product     ***%s", $prod, $::EOL);
                } elsif ($status =~ /^NO REPORT( .*)?$/i) {
                    printf(RESULTV "%s    | *** no report - see problem list              ***%s", $prod, $::EOL);
                    printf(RESULTF "%s    | *** no report - see problem list              ***%s", $prod, $::EOL);
                } elsif ($status =~ /^-S1( .*)?$/i) {
                    printf(RESULTV "%s*   | ** no report yet                               **%s", $prod, $::EOL);
                    printf(RESULTF "%s*   | ** no report yet                               **%s", $prod, $::EOL);
                } elsif ($status =~ /^S1( .*)?$/i) {
                    printf(RESULTV "%s*   | ** no evaluation yet                           **%s", $prod, $::EOL);
                    printf(RESULTF "%s*   | ** no evaluation yet                           **%s", $prod, $::EOL);
                } else {
                    if ($status =~ /^OK S\dEQ?/i) {
                        printf(RESULTV "%s    ", $prod);
                        printf(RESULTF "%s    ", $prod);
                    } elsif ($status =~ /^S\dE( .*)?$/i) {
                        printf(RESULTV "%s*   ", $prod);
                        printf(RESULTF "%s*   ", $prod);
                    } elsif ($status =~ /^-?S(\d)( .*)?$/i) {
                        my $postscanNr = $1 - 1;
                        printf(RESULTV "%s*p%s ", $prod, $postscanNr);
                        printf(RESULTF "%s*p%s ", $prod, $postscanNr);
                    } else {
                        printf(RESULTV "%s*   ", $prod);
                        printf(RESULTF "%s*   ", $prod);
                    }
                    foreach my $pack (@packList) {
                        my $prodEval = $prod . $pack;
                        if ($id ne "") {
                            $prodEval .= "_" . $id;
                        }
                        my @data = @{$::data{"result"}->{$os}->{$tb}->{$prodEval}};
                        if ($data[0] eq "---") {
                            print(RESULTV "  NOT FOUND ");
                            print(RESULTF "  NOT FOUND ");
                            $ret = 1;
                        } else {
                            printf(RESULTV "| %4d %5.1f%% ", $data[1], $data[2]);
                            printf(RESULTF "| %4d %5.1f%% ", $data[7], $data[8]);
                        }
                    }
                    print(RESULTV "|", $::EOL);
                    print(RESULTF "|", $::EOL);
                }
            } # end of LOOP_PROD
            print(RESULTV "-------+");
            print(RESULTF "-------+");
            foreach my $pack (@packList) {
                print(RESULTV "-------------+");
                print(RESULTF "-------------+");
            }
            print(RESULTV $::EOL);
            print(RESULTF $::EOL);
        }
        close(RESULTV);
        close(RESULTF);
    }

    return $ret;
}


sub createPackedArchResults {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    LOOP_PACKARCH: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        ::log("creating packed 'all files' result table for: $os - $list\n", 1);
        my $tempList = $list;
        substr($tempList, 7, 1) = "A";
        my $resultFile = $::ResultDir . $::sep . $tempList . "." . $os;

        open(RESULT, "> $resultFile") or do {
            ::log("ERROR creating $resultFile: $!\n", 1);
            ::log("!!! Result generation for $tb stopped !!!\n\n", 4);
            next LOOP_PACKARCH;
        };
        binmode(RESULT);

        # write result table header
        my $header = sprintf('Table %3s.%1s : General detection in "%s" '.
                             'archives under %s%s',
                             $os, substr($tb, 0, 1), $::listsAll{$tb}->{"name"},
                             $::osListAll{$os}, $::EOL);
        $header = ::indentString($header, ": ", 78);
        print(RESULT $header, "="x78, $::EOL);
        print(RESULT "Product     Total number of detected files in all archives",
                     $::EOL);
        print(RESULT "-"x60, $::EOL);
        printf(RESULT "Testbed     %6d 100.0%%" . " "x28 . "%s",
                      $::listsAll{$tb}->{"packed"}->{"sumF"}, $::EOL);
        print(RESULT "-"x60, $::EOL);

        LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
            # check for not tested combinations
            foreach my $skip (@::noTestListAll) {
                if ($skip eq "$os $tb $prod") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - $prod\n", 2);
                    }
                    next LOOP_PROD;
                }
            }
            my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
            my $status = ::getStatus($statFile);
            if ($status eq "") {
                ::log("ERROR getting status for $os - $tb - $prod\n", 2);
                $ret = 1;
                print(RESULT $prod . "*   | *** ERROR ***" . $::EOL);
            } elsif ($status =~ /^NO TEST( .*)?$/i) {
                # it makes no sense here to include the product in the table
            } elsif ($status =~ /^NO SUPPORT( .*)?$/i) {
                printf(RESULT "%s      *** testbed not supported by this product     ***%s", $prod, $::EOL);
            } elsif ($status =~ /^NO REPORT( .*)?$/i) {
                printf(RESULT "%s      *** no report - see problem list              ***%s", $prod, $::EOL);
            } elsif ($status =~ /^S1( .*)?$/i) {
                printf(RESULT "%s*     ** no evaluation yet                           **%s", $prod, $::EOL);
            } elsif ($status =~ /^-S1( .*)?$/i) {
                printf(RESULT "%s*     ** no report yet                               **%s", $prod, $::EOL);
            } else {
                if ($status =~ /^OK S\dEQ?/i) {
                    printf(RESULT "%s         ", $prod);
                } elsif ($status =~ /^S\dE( .*)?$/i) {
                    printf(RESULT "%s*        ", $prod);
                } elsif ($status =~ /^-?S(\d)( .*)?$/i) {
                    my $postscanNr = $1 - 1;
                    printf(RESULT "%s*p%s      ", $prod, $postscanNr);
                } else {
                    printf(RESULT "%s*        ", $prod);
                }
                my $prodEval = $prod . $::FileExtArchiveFiles;
                if ($id ne "") {
                    $prodEval .= "_" . $id;
                }
                my @data = @{$::data{"result"}->{$os}->{$tb}->{$prodEval}};
                if ($data[0] eq "---") {
                    print(RESULT "NOT FOUND ");
                    $ret = 1;
                } else {
                    printf(RESULT "%6d %5.1f%%%s", $data[7], $data[8], $::EOL);
                }
            }
        } # end of LOOP_PROD
        print(RESULT "-"x60, $::EOL);
        close(RESULT);
    }
    return $ret;
}


sub createPackedLossResults {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $refTb = $_[2];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    # list of all compression programs used in this testbed
    my %packPrgs = %{$::lists{$tb}->{"packed"}->{"prg"}};
    LOOP_PACKLOSS: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        ::log("creating packed 'loss' result table for: $os - $list\n", 1);
        my $tempList = $list;
        substr($tempList, 7, 1) = "L";
        my $resultFile = $::ResultDir . $::sep . $tempList . "." . $os;

        open(RESULT, "> $resultFile") or do {
            ::log("ERROR creating $resultFile: $!\n", 1);
            ::log("!!! Result generation for $tb stopped !!!\n\n", 4);
            next LOOP_PACKLOSS;
        };
        binmode(RESULT);

        # write result table header
        my $header = sprintf('Table %3s.%1s : Detection loss in "%s" archives '.
                             'under %s%s', $os, substr($tb, 0, 1),
                             $::lists{$tb}->{"evals"}->{$list}->{"name"},
                             $::osList{$os}, $::EOL);
        $header = ::indentString($header, ": ", 78);
        print(RESULT $header, "="x78, $::EOL);
        print(RESULT "        Number of files not detected in archives".
                     " (but uncompressed)");

        # Divide all compression programs into sections, each containing
        # 5 programs. These are used to create better formatted tables
        my $sec = 0;
        my $nr = 0;
        my %packSec = ();
        foreach my $pack (sort keys %packPrgs) {
            push @{$packSec{$sec}}, $pack;
            $nr++;
            if (($nr % 5) == 0) {
                $sec++;
            }
        }

        foreach my $i (sort keys %packSec) {
            my @packList = @{$packSec{$i}};
            print(RESULT $::EOL, "-------+");
            foreach my $pack (@packList) {
                print(RESULT "-------------+");
            }
            print(RESULT $::EOL, "       |");
            foreach my $pack (@packList) {
				my $packname = $::packListAll{$pack}->{"name"};
				if (length($packname) > 9) {
                    $packname = substr($pack, 0, 9);
                }
                printf(RESULT "  %-9s  |", $packname);
            }
            print(RESULT $::EOL, "-------+");
            foreach my $pack (@packList) {
                print(RESULT "-------------+");
            }
            print(RESULT $::EOL);

            LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                # check for not tested combinations
                foreach my $skip (@::noTestListAll) {
                    if ($skip eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 2);
                        }
                        next LOOP_PROD;
                    }
                }
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $status = ::getStatus($statFile);
                my $refStatFile = $::EvalBase . $::sep . $os . $::sep .
                                  $refTb . $::sep . $prod . $::FileExtStatus;
                my $refStatus = ::getStatus($refStatFile);
                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod\n", 2);
                    $ret = 1;
                } elsif ($refStatus eq "") {
                    ::log("ERROR getting status for $os - $refTb - $prod\n", 2);
                    $ret = 1;
                } elsif ($status =~ /^NO TEST( .*)?$/i
                         or $refStatus =~ /^NO TEST( .*)?$/i) {
                    # it makes no sense here to include the product in the table
                } elsif ($status =~ /^NO SUPPORT( .*)?$/i
                         or $refStatus =~ /^NO SUPPORT( .*)?$/i) {
                    printf(RESULT "%s    | *** testbed not supported by this product     ***%s", $prod, $::EOL);
                } elsif ($status =~ /^NO REPORT( .*)?$/i
                         or $refStatus =~ /^NO REPORT( .*)?$/i) {
                    printf(RESULT "%s    | *** no report - see problem list              ***%s", $prod, $::EOL);
                } elsif ($status =~ /^S1( .*)?$/i
                         or $refStatus =~ /^S1( .*)?$/i) {
                    printf(RESULT "%s*   | ** no evaluation yet                           **%s", $prod, $::EOL);
                } elsif ($status =~ /^-S1( .*)?$/i
                         or $refStatus =~ /^-S1( .*)?$/i) {
                    printf(RESULT "%s*   | ** no report yet                               **%s", $prod, $::EOL);
                } else {
                    if ($status =~ /^OK S\dEQ?/i
                        and $refStatus =~ /^OK S\dEQ?/i) {
                        printf(RESULT "%s    ", $prod);
                    } else {
                        printf(RESULT "%s*   ", $prod);
                    }
                    foreach my $pack (@packList) {
                        my $prodEval = $prod . $pack;
                        my $prodRef = $prod;
                        if ($id ne "") {
                            $prodEval .= "_" . $id;
                            $prodRef .= "_" . $id;
                        }
                        if ($::debug > 2) {
                            ::log("creating loss results for: $os - $tb (ref:".
                                  " $refTb) - $prodEval (ref: $prodRef)\n", 3);
                        }
                        my $files = ${$::data{"result"}->{$os}->{$tb}->{$prodEval}}[7];
                        my $refFiles = ${$::data{"result"}->{$os}->{$refTb}->{$prodRef}}[7];
                        if ($files == -1 or $refFiles == -1) {
                            print(RESULT "  NOT FOUND ");
                            $ret = 1;
                        } else {
                            my $lossFiles = $refFiles - $files;
                            my $lossPerc;
                            if ($lossFiles < 0) {
                                $lossPerc = "***** ";
                            } elsif ($refFiles > 0) {
                                $lossPerc = $lossFiles * 100 / $refFiles;
                                $lossPerc = sprintf("%5.1f%%", $lossPerc);
                                if ($lossPerc eq "100.0%" and $lossFiles < $refFiles) {
                                    $lossPerc = "100.~%";
                                } elsif ($lossPerc eq "  0.0%" and $lossFiles > 0) {
                                    $lossPerc = "  0.~%";
                                }
                            } else {
                                $lossPerc = "----- ";
                            }
                            printf(RESULT "| %4d %6s ", $lossFiles, $lossPerc);
                        }
                    }
                    print(RESULT "|", $::EOL);
                }
            } # end of LOOP_PROD
            print(RESULT "-------+");
            foreach my $pack (@packList) {
                print(RESULT "-------------+");
            }
            print(RESULT $::EOL);
        }
        close(RESULT);
    }

    return $ret;
}



sub createCSVLists {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    LOOP_EVAL: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        ::log("creating result table for: $os - $list\n", 1);
        my $resultFile = $::ResultCSVDir . $::sep . $list . "." . $os;

        open(RESULT, "> $resultFile") or do {
            ::log("ERROR creating $resultFile: $!\n", 1);
            ::log("!!! Result generation of $list stopped !!!\n\n", 4);
            $ret = 1;
            next LOOP_EVAL;
        };
        binmode(RESULT);

        LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
            # check for not tested combinations
            foreach (@::noTestListAll) {
                if ($_ eq "$os $tb $prod") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - $prod\n", 2);
                    }
                    next LOOP_PROD;
                }
            }
            # define name of product related files
            my $prodEval = $prod;
            if ($id ne "") {
                $prodEval .= "_" . $id;
            }
            my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
            my @data = @{$::data{"result"}->{$os}->{$tb}->{$prodEval}};
            $ret += processProductCSV($prod, $statFile, @data);
        }
        close(RESULT);
    }

    return $ret;
}


sub createPackedCSVLists {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    # list of all compression programs used in this testbed
    my %packPrgs = %{$::lists{$tb}->{"packed"}->{"prg"}};
    foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        LOOP_PACK: foreach my $pack (sort keys %packPrgs, $::FileExtArchiveFiles) {
            ::log("creating result table for: $os - $list - $pack\n", 1);
            my $tempList = $list;
            substr($tempList, 5, 3) = $pack;
            my $resultFile = $::ResultCSVDir . $::sep . $tempList . "." . $os;

            open(RESULT, "> $resultFile") or do {
                ::log("ERROR creating $resultFile: $!\n", 1);
                ::log("!!! Result generation of $list stopped !!!\n\n", 4);
                $ret = 1;
                next LOOP_PACK;
            };
            binmode(RESULT);

            LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 2);
                        }
                        next LOOP_PROD;
                    }
                }
                # define name of product related files
                my $prodEval = $prod . $pack;
                if ($id ne "") {
                    $prodEval .= "_" . $id;
                }
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my @data = @{$::data{"result"}->{$os}->{$tb}->{$prodEval}};
                $ret += processProductCSV($prod, $statFile, @data);
            }
            close(RESULT);
        }   #end of foreach LOOP_EVAL
    }

    return $ret;
}


sub processProductCSV {
    my $ret = 0;
    my $prod = shift @_;
    my $statFile = shift @_;
    my @data = @_;
    # get status of current product
    my $status = ::getStatus($statFile);

    if ($status eq "") {
        ::log("ERROR getting status for $prod\n", 2);
        $ret = 1;
    } elsif ($status =~ /^NO TEST( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } elsif ($status =~ /^NO SUPPORT( .*)?$/i) {
        print(RESULT "$prod;-1;-1;-1;-1;-1;-1;-1;-1", $::EOL);
    } elsif ($status =~ /^NO REPORT( .*)?$/i) {
        print(RESULT "$prod;-1;-1;-1;-1;-1;-1;-1;-1", $::EOL);
    } elsif ($status =~ /^-S1( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } elsif ($status =~ /^S1( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } else {
        # we make no difference between final and preliminary results
        my $out = join(";", @data);
        # Because database import requires numeric values, values 
        # represented as 100.~ or 0.~ in ASCII result tables, are
        # represented here as 100.1 and 0.01. These are NOT the real 
        # values but only numeric representations to trigger database 
        # scripts
		$out =~ s/100\.~/100.1/g;
		$out =~ s/0\.~/0.01/g;
        print(RESULT $out, $::EOL);
    }

    return $ret;
}


sub createPackedLossCSVLists {
    my $ret = 0;
    my $os = $_[0];
    my $tb = $_[1];
    my $refTb = $_[2];
    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
    # list of all compression programs used in this testbed
    my %packPrgs = %{$::lists{$tb}->{"packed"}->{"prg"}};
    foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        LOOP_PACK: foreach my $pack (sort keys %packPrgs, $::FileExtArchiveFiles) {
            ::log("creating loss CSV table for: $os - $list - $pack\n", 1);
            my $tempList = $list;
            substr($tempList, 5, 3) = $pack;
            my $resultFile = $::ResultLossCSVDir . $::sep . $tempList
	                     . "." . $os;

            open(RESULT, "> $resultFile") or do {
                ::log("ERROR creating $resultFile: $!\n", 1);
                ::log("!!! Result generation of $list stopped !!!\n\n", 4);
                $ret = 1;
                next LOOP_PACK;
            };
            binmode(RESULT);

            LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 2);
                        }
                        next LOOP_PROD;
                    }
                }
                # define name of product related files
                my $prodEval = $prod . $pack;
                my $prodRef = $prod;
                if ($id ne "") {
                    $prodEval .= "_" . $id;
                    $prodRef .= "_" . $id;
                }
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $refStatFile = $::EvalBase . $::sep . $os . $::sep .
                                  $refTb . $::sep . $prod . $::FileExtStatus;
                $ret += processProductLossCSV($prod, $prodRef,
		                              $statFile, $refStatFile,
					      $::data{"result"}->{$os}->{$tb}->{$prodEval},
					      $::data{"result"}->{$os}->{$refTb}->{$prodRef}
					      );
            }
            close(RESULT);
        }   #end of foreach LOOP_PACK
    }

    return $ret;
}


sub processProductLossCSV {
    my $ret = 0;
    my $prod = $_[0];
    my $prodRef = $_[1];
    my $statFile = $_[2];
    my $refStatFile = $_[3];
    my @data = @{$_[4]};
    my @refData = @{$_[5]};
    my @lossData = ();
    # get status of current product
    my $status = ::getStatus($statFile);
    my $refStatus = ::getStatus($refStatFile);

    if ($status eq "" or $refStatus eq "") {
        ::log("ERROR getting status for $prod\n", 2);
        $ret = 1;
    } elsif ($status =~ /^NO TEST( .*)?$/i
             or $refStatus =~ /^NO TEST( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } elsif ($status =~ /^NO SUPPORT( .*)?$/i
             or $refStatus =~ /^NO SUPPORT( .*)?$/i) {
        print(RESULT "$prod;-1;-1;-1;-1;-1;-1;-1;-1", $::EOL);
    } elsif ($status =~ /^NO REPORT( .*)?$/i
             or $refStatus =~ /^NO REPORT( .*)?$/i) {
        print(RESULT "$prod;-1;-1;-1;-1;-1;-1;-1;-1", $::EOL);
    } elsif ($status =~ /^-S1( .*)?$/i
             or $refStatus =~ /^-S1( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } elsif ($status =~ /^S1( .*)?$/i
             or $refStatus =~ /^S1( .*)?$/i) {
        # it makes no sense here to include the product in the table
    } else {
	push @lossData, $prod;
	for (my $i=1; $i<=7; $i+=2) {
	    my $dataAbs = $data[$i];
	    my $refAbs = $refData[$i];
	    my $lossAbs;
	    my $lossPerc;
	    if ($dataAbs == -1 or $refAbs == -1) {
            $lossAbs = "-1";
            $lossPerc = "-1";
	    } else {
            $lossAbs = $refAbs - $dataAbs;
            if ($lossAbs < 0 or ($lossAbs == 0 and $refAbs == 0)) {
                    $lossPerc = "0.0";
            } elsif ($refAbs > 0) {
        	    $lossPerc = $lossAbs * 100 / $refAbs;
        	    $lossPerc = sprintf("%.1f", $lossPerc);
        	    if ($lossPerc eq "100.0" and $lossAbs < $refAbs) {
					# This is NOT the real value but a number representing all
					# values between 99.9% and 100.0%
					# This is represented as 100.~ in ASCII result tables,
					# but database import requires a numeric value.
                    $lossPerc = "100.1";
        	    } elsif ($lossPerc eq "  0.0" and $lossAbs > 0) {
        	    	# This is NOT the real value (see comment above)
                    $lossPerc = "  0.01";
        	    }
            } else {
        	    $lossPerc = "-1";
            }
	    }
	    push @lossData, $lossAbs;
	    push @lossData, $lossPerc;
	}

        # we make no difference between final and preliminary results
        my $out = join(";", @lossData);
        print(RESULT $out, $::EOL);
    }
    return $ret;
}


# Make all result data available via the dynamic data hash %data
# This uses only the keys "result" and "resultLine"
sub getData {
    # force quiet flag for this loop
    # (this must overide the global setting because of function calls)
    my $oldquiet = $::quiet;
    if (not $::quiet) {
        $::quiet = 1;
    }
    foreach my $os (sort keys %::osList) {
        # this must be done for all testbeds to make sure that the
        # reference testbed data (for packed testbeds) is available
        foreach my $tb (sort keys %::listsAll) {
            my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
            foreach my $list (keys %{$::listsAll{$tb}->{"evals"}}) {
                my $id = $::listsAll{$tb}->{"evals"}->{$list}->{"id"};
                ::log("reading result data for: $os - $tb - $list\n", 1);
                LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
                    # check for not tested combinations
                    foreach my $skip (@::noTestListAll) {
                        if ($skip eq "$os $tb $prod") {
                            if ($::debug) {
                                ::log("not tested: $os - $tb - $prod\n", 2);
                            }
                            next LOOP_PROD;
                        }
                    }
                    if (exists $::listsAll{$tb}->{"packed"}) {
                        my %packPrgs = %{$::listsAll{$tb}->{"packed"}->{"prg"}};
                        foreach my $pack (sort keys %packPrgs, $::FileExtArchiveFiles) {
                            my $prodEval = $prod . $pack;
                            if ($id ne "") {
                                $prodEval .= "_" . $id;
                            }
                            my $datFile = $EvalDir . $::sep . $prodEval .
                                          "." . $::FileExtResult;
                            my @result = ::getDataArray($datFile);
                            my $resultLine = ::getDataLine($prod, $datFile);
                            @{$::data{"result"}->{$os}->{$tb}->{$prodEval}} = @result;
                            $::data{"resultLine"}->{$os}->{$tb}->{$prodEval} =  $resultLine;
                            if ($::debug > 2) {
                                ::log("$os-$list-$prod-$pack: line='$resultLine' - array='@result'\n");
                            }
                        }
                    } else {
                        my $prodEval = $prod;
                        if ($id ne "") {
                            $prodEval .= "_" . $id;
                        }
                        my $datFile = $EvalDir . $::sep . $prodEval . "." .
                                      $::FileExtResult;
                        my @result = ::getDataArray($datFile);
                        my $resultLine = ::getDataLine($prod, $datFile);
                        @{$::data{"result"}->{$os}->{$tb}->{$prodEval}} = @result;
                        $::data{"resultLine"}->{$os}->{$tb}->{$prodEval} =  $resultLine;
                        if ($::debug > 2) {
                            ::log("$os-$list-$prod: line='$resultLine' - array='@result'\n");
                        }
                    }
                }
            }
        }
    }
    # restore previous quiet flag
    $::quiet = $oldquiet;
}


# return a true value
1;