# Rpt_comp.pm
# implements 'report compare' command
#
# v0.15, 2004-03-29
# Copyright (c) Michel Messerschmidt 2002-2004
#
package Avtc::Rpt_comp;
use strict;
use Exporter;
use File::Copy;

@Avtc::Rpt_comp::ISA = qw(Exporter);
@Avtc::Rpt_comp::EXPORT = qw(reportCompare checkCompare);


sub reportCompare {
    my $ret = doReportCompare("all");
    $ret += doReportCompare("win");
    return $ret;
}

sub doReportCompare {
    my $ret = 0;
    my $whichOs = $_[0];
    my @cmpOs = ();
    my ($extCmpF, $extCmpV);
    # choose which systems should be compared
    if ($whichOs eq "all") {
        # use the following line to compare all operating systems
        @cmpOs = (sort keys %::osListAll);
        $extCmpF = $::FileExtCmpF;
        $extCmpV = $::FileExtCmpV;
    } elsif ($whichOs eq "win") {
        # use this loop to compare only windows systems
        foreach my $os (sort keys %::osListAll) {
            if ($os =~ /^W/i) {
                push @cmpOs, $os;
            }
        }
        $extCmpF = $::FileExtCmpWinF;
        $extCmpV = $::FileExtCmpWinV;
    } else {
        ::log("ERROR: invalid parameter for 'doReportCompare' function\n", 0);
        return 1;
    }
    foreach my $tb (sort keys %::lists) {
        LOOP_EVAL: foreach my $list (keys %{$::lists{$tb}->{"evals"}}) {
            if (exists $::lists{$tb}->{"packed"}) {
                my %packPrgs = %{$::listsAll{$tb}->{"packed"}->{"prg"}};
                LOOP_PACK: foreach my $pack (sort keys %packPrgs,
                                             $::FileExtArchiveFiles) {
                    my $pList = $list;
                    substr($pList, 5, 3) = $pack;
                    ::log("processing list: $pList  -  tb: $tb\n", 2);
                    processList($tb, $list, $pList, $pack, $extCmpF,
                                $extCmpV, \@cmpOs) and do {
                        ::log("!!! Result generation of $pList stopped !!!".
                              "\n\n", 3);
                        $ret = 1;
                        next LOOP_PACK;
                    };
                }
            } else {
                ::log("processing list: $list  -  tb: $tb\n", 2);
                processList($tb, $list, $list, "", $extCmpF, $extCmpV,
                            \@cmpOs) and do {
                    ::log("!!! Result generation of $list stopped !!!\n\n",
                          3);
                    $ret = 1;
                    next LOOP_EVAL;
                };
            }
        } #end of LOOP_EVAL
    }
    return $ret;
}


sub processList {
    my $ret = 0;
    my $tb = $_[0];        # testbed (used for header output)
    my $list = $_[1];      # evaluation list (used for header output)
    my $pList = $_[2];     # name of packed evaluation list or equal to $list
                           # if testbed not packed (used for output filename)
    my $pack = $_[3];      # code of current packer or "" if testbed is not
                           # packed (used for input data filename)
    my $extCmpF = $_[4];   # Filename extension for file comparison results
    my $extCmpV = $_[5];   # Filename extension for virus comparison results
    my @cmpOs = @{$_[6]};  # list of all compared operating system
    my %results;
    my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};

    LOOP_PROD: foreach my $prod (sort keys %::prodListAll) {
        my $prodEval = $prod . $pack;
        if ($id ne "") {
            $prodEval .= "_" . $id;
        }
        LOOP_OS: foreach my $os (@cmpOs) {
            # check for not tested combinations
            foreach (@::noTestListAll) {
                if ($_ eq "$os $tb $prod") {
                    if ($::debug) {
                        ::log("not tested: $os - $tb - $prod\n", 3);
                    }
                    $results{$prod}->{$os}->{"V"} = "-----";
                    $results{$prod}->{$os}->{"VP"} = "------";
                    $results{$prod}->{$os}->{"F"} = "-----";
                    $results{$prod}->{$os}->{"FP"} = "------";
                    next LOOP_OS;
                }
            }
            my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
            my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
            my $datFile = $EvalDir . $::sep . $prodEval . "." .
                          $::FileExtResult;
            # check status and decide if result data is available
            my $status = ::getStatus($statFile);

            if ($status eq "") {
                ::log("ERROR getting status for $os - $tb - $prod\n", 3);
                $ret = 1;
                $results{$prod}->{$os}->{"V"} = "ERROR";
                $results{$prod}->{$os}->{"VP"} = "ERROR";
                $results{$prod}->{$os}->{"F"} = "ERROR";
                $results{$prod}->{$os}->{"FP"} = "ERROR";
                next LOOP_OS;
            } elsif ($status =~ /^NO/i) {
                $results{$prod}->{$os}->{"V"} = "-----";
                $results{$prod}->{$os}->{"VP"} = "------";
                $results{$prod}->{$os}->{"F"} = "-----";
                $results{$prod}->{$os}->{"FP"} = "------";
                next LOOP_OS;
            } elsif ($status =~ /^-S1/i) {
                $results{$prod}->{$os}->{"V"} = "TODO";
                $results{$prod}->{$os}->{"VP"} = "TODO";
                $results{$prod}->{$os}->{"F"} = "TODO";
                $results{$prod}->{$os}->{"FP"} = "TODO";
                $results{$prod}->{"noQA"} = 1;
                next LOOP_OS;
            }
            elsif ($status !~ /^OK/i) {
                # if results are not final, set a flag for the output
                $results{$prod}->{"noQA"} = 1;
            }

            my @tempData = ::getDataArray($datFile);
            if ($id eq "F") {
                # false positive lists have no 'viruses' data
                $tempData[7] = $tempData[1];
                $tempData[8] = $tempData[2];
            }
            $tempData[2] = $tempData[2] . "%";
            $tempData[8] = $tempData[8] . "%";
            if ($tempData[1] eq "-1") { $tempData[1] = "NOT"; }
            if ($tempData[2] eq "-1%") { $tempData[2] = "FOUND"; }
            if ($tempData[7] eq "-1") { $tempData[7] = "NOT"; }
            if ($tempData[8] eq "-1%") { $tempData[8] = "FOUND"; }
            $results{$prod}->{$os}->{"V"} = $tempData[1];
            $results{$prod}->{$os}->{"VP"} = $tempData[2];
            $results{$prod}->{$os}->{"F"} = $tempData[7];
            $results{$prod}->{$os}->{"FP"} = $tempData[8];
        }
    }  #end of foreach LOOP_PROD
    $ret += printComp($tb, $list, $pList, $pack, $extCmpF, $extCmpV,
                      \@cmpOs, \%results);
    return $ret;
}


sub printComp {
    my $ret = 0;
    my $tb = $_[0];
    my $list = $_[1];
    my $pList = $_[2];
    my $pack = $_[3];
    my $extCmpF = $_[4];
    my $extCmpV = $_[5];
    my @cmpOs = @{$_[6]};
    my %results = %{$_[7]};
    my $resultF = $::ResultDir . $::sep . $pList . "." . $extCmpF;
    my $resultV = $::ResultDir . $::sep . $pList . "." . $extCmpV;

    foreach my $prod (sort keys %::prodListAll) {
        my $firstOS = $cmpOs[0];
        my $lastF = $results{$prod}->{$firstOS}->{"F"};
        my $lastV = $results{$prod}->{$firstOS}->{"V"};
        my $changeF = 0;
        my $changeV = 0;
        my $count = 0;
        foreach my $os (@cmpOs) {
            my $tempF = $results{$prod}->{$os}->{"F"};
            my $tempV = $results{$prod}->{$os}->{"V"};
            if (($tempF ne $lastF and $tempF ne "-----" and $lastF ne "-----"
                 and $tempF ne "ERROR" and $lastF ne "ERROR")
                or ($tempF eq "TODO")) {
                $changeF = 1;
            }
            if (($tempV ne $lastV and $tempV ne "-----" and $lastV ne "-----"
                 and $tempV ne "ERROR" and $lastV ne "ERROR")
               or ($tempV eq "TODO")) {
                $changeV = 1;
            }
            if (($tempF ne "-----" and $tempF ne "ERROR")
                or ($tempV ne "-----" and $tempV ne "ERROR")) {
                $count++;
            }
            $lastF = $tempF;
        }
        $results{$prod}->{"matchF"} = 1 unless $changeF;
        $results{$prod}->{"matchV"} = 1 unless $changeV;
        if ($count < 2) {
            $results{$prod}->{"noCount"} = 1;
        }
    }

    # computer the correct distances for header texts
    my $lineLength = ($#cmpOs + 1) * 15;
    my $lineBack = int(($lineLength - 27) / 2);
    my $lineFront = $lineLength - 27 - $lineBack;

    open(RESULTF, "> $resultF") or do {
        ::log("ERROR creating $resultF: $!\n", 3);
        return 1;
    };
    binmode(RESULTF);

    # write result table header
    if ($pack eq $::FileExtArchiveFiles) {
        printf(RESULTF 'Table Comp.%1s : Detection Quality of "%s" Samples '.
               '%s               in packed achives%s', substr($tb,0,1),
               $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL, $::EOL);
    } elsif ($pack) {
        printf(RESULTF 'Table Comp.%1s : Detection Quality of "%s" Samples '.
               '%s               packed with %s%s', substr($tb,0,1),
               $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL,
               $::packListAll{$pack}->{"name"}, $::EOL);
    } else {
        printf(RESULTF 'Table Comp.%1s : Detection Quality of "%s" Samples '.
               '%s', substr($tb,0,1),
               $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL);
    }
    print(RESULTF "=" x 78, $::EOL);
    print(RESULTF "        ");
    print(RESULTF " " x $lineFront, "==== Detection Quality ====",
                  " " x $lineBack);
    print(RESULTF "  results", $::EOL);
    print(RESULTF "Product ");
    foreach my $os (@cmpOs) {
        printf(RESULTF "  %13s", $::osListAll{$os});
    }
    print(RESULTF "   match", $::EOL);
    print(RESULTF "-" x 78, $::EOL);
    foreach my $prod (sort keys  %::prodListAll) {
        if (exists $results{$prod}->{"noCount"}) {
            next;
        }
        print(RESULTF $prod);
        if (exists $results{$prod}->{"noQA"}) {
            print(RESULTF "*    ");
        } else {
            print(RESULTF "     ");
        }
        foreach my $os (@cmpOs) {
            printf(RESULTF "  %6s %6s", $results{$prod}->{$os}->{"F"},
                           $results{$prod}->{$os}->{"FP"});
        }
        if (exists $results{$prod}->{"matchF"}) {
            print(RESULTF "     *");
        }
        print(RESULTF $::EOL);
    }
    print(RESULTF "-" x 78, $::EOL);

    close(RESULTF);

    # false positives lists have no results for 'viruses'
    if ($list !~ /^..F.....$/i) {
        open(RESULTV, "> $resultV") or do {
            ::log("ERROR creating $resultV: $!\n", 3);
            return 1;
        };
        binmode(RESULTV);

        # write result table header
        if ($pack eq $::FileExtArchiveFiles) {
            printf(RESULTV 'Table Comp.%1s : Detection Quality of "%s" '.
                   'Viruses %s               in packed achives%s',
                   substr($tb,0,1),
                   $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL, $::EOL);
        } elsif ($pack) {
            printf(RESULTV 'Table Comp.%1s : Detection Quality of "%s" '.
                   'Viruses %s               packed with %s%s',
                   substr($tb,0,1),
                   $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL,
                   $::packListAll{$pack}->{"name"}, $::EOL);
        } else {
            printf(RESULTV 'Table Comp.%1s : Detection Quality of "%s" '.
                   'Viruses %s', substr($tb,0,1),
                   $::listsAll{$tb}->{"evals"}->{$list}->{"name"}, $::EOL);
        }
        print(RESULTV "=" x 78, $::EOL);
        print(RESULTV "        ");
        print(RESULTV " " x $lineFront, "==== Detection Quality ====",
                      " " x $lineBack);
        print(RESULTV "  results", $::EOL);
        print(RESULTV "Product ");
        foreach my $os (@cmpOs) {
            printf(RESULTV "  %13s", $::osListAll{$os});
        }
        print(RESULTV "   match", $::EOL);
        print(RESULTV "-" x 78, $::EOL);
        foreach my $prod (sort keys  %::prodListAll) {
            if (exists $results{$prod}->{"noCount"}) {
                next;
            }
            print(RESULTV $prod);
            if (exists $results{$prod}->{"noQA"}) {
                print(RESULTV "*    ");
            } else {
                print(RESULTV "     ");
            }
            foreach my $os (@cmpOs) {
                printf(RESULTV "  %6s %6s", $results{$prod}->{$os}->{"V"},
                               $results{$prod}->{$os}->{"VP"});
            }
            if (exists $results{$prod}->{"matchV"}) {
                print(RESULTV "     *");
            }
            print(RESULTV $::EOL);
        }
        print(RESULTV "-" x 78, $::EOL);
        close(RESULTV);
    }
    return $ret;
}



# create a file with ALL single results to compare product
# behaviour on all systems. This is used by the quality assurance
# to find inconsistent test procedures
sub checkCompare {
    my $ret = 0;
    my $checkFile = $::ReportDir . $::sep . $::FileCmpAll;
    open(CHECK, "> $checkFile") or do {
        ::log("ERROR creating $checkFile: $!\n", 1);
        return 1;
    };
    binmode(CHECK);

    foreach my $tb (sort keys %::listsAll) {
        if (exists $::listsAll{$tb}->{"packed"}) {
            $ret = compAllPackedLists($tb);
        } else {
            $ret = compAllLists($tb);
        }
    }
    close(CHECK);

    return $ret;
}


sub compAllLists {
    my $ret = 0;
    my $tb = $_[0];
    foreach my $list (keys %{$::listsAll{$tb}->{"evals"}}) {
        ::log("processing list: $list  -  tb: $tb\n", 2);
        my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
        print(CHECK $::EOL,$::EOL,"Testbed: ",$tb,"  -  evallist: ",$list,
              $::EOL);
        printf(CHECK "         %6d 100.0%%                        %6d ".
               "100.0%%%s", $::lists{$tb}->{"evals"}->{$tb}->{"sumV"},
               $::lists{$tb}->{"evals"}->{$tb}->{"sumF"}, $::EOL);
        print(CHECK "-----------------------------------------------------",
              "-------", $::EOL);

        foreach my $prod (sort keys %::prodListAll) {
            my $prodEval = $prod;
            if ($id ne "") {
                $prodEval .= "_" . $id;
            }
            my $lastData = "";
            LOOP_OS: foreach my $os (sort keys %::osListAll) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("not tested: $os - $tb - $prod\n", 3);
                        }
                        next LOOP_OS;
                    }
                }

                my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $status = ::getStatus($statFile);
                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod\n", 2);
                    $ret = 1;
                    next LOOP_OS;
                } elsif ($status =~ /^NO/i or $status =~ /^-?S1( .*)?$/i) {
                    # there are no results yet for status '-S1' and 'S1'
                    next LOOP_OS;
                }
                # else: there are results for all other status codes
                my $datFile = $EvalDir . $::sep . $prodEval . "." .
                              $::FileExtResult;
                my $dataLine = ::getDataLine($prod, $datFile);
                my $outLine = $dataLine;
                if ($status =~ /^S1E/i) {
                    substr($outLine, 3, 1) = "*";
                } elsif ($status =~ /^-?S(\d)E?/i) {
                    my $postscanNr = $1 - 1;
                    substr($outLine, 3, 3) = "*p" . $postscanNr;
                }

                if ($lastData eq "") {
                    print(CHECK $::EOL);
                }
                print(CHECK $outLine,"  -  ",$os);
                if ($lastData ne "" and $dataLine ne $lastData) {
                    print(CHECK " !!!");
                }
                print(CHECK $::EOL);
                $lastData = $dataLine;
            }
        }
    }
    return $ret;
}


sub compAllPackedLists {
    my $ret = 0;
    my $tb = $_[0];
    my %packPrgs = %{$::listsAll{$tb}->{"packed"}->{"prg"}};
    foreach my $pack (sort keys %packPrgs, $::FileExtArchiveFiles) {
		my $packname = $::packListAll{$pack}->{"name"};
        foreach my $list (keys %{$::listsAll{$tb}->{"evals"}}) {
            my $tempList = $list;
            substr($tempList, 5, 3) = $pack;
            ::log("processing list: $tempList  -  tb: $tb\n", 2);
            my $id = $::lists{$tb}->{"evals"}->{$list}->{"id"};
            print(CHECK $::EOL,$::EOL,"Testbed: ",$tb,"  -  evallist: ",
                  $list,"  -  pack:",$packname,$::EOL);
            printf(CHECK "         %6d 100.0%%                        %6d ".
                   "100.0%%%s", $::lists{$tb}->{"evals"}->{$tb}->{"sumV"},
                   $::lists{$tb}->{"evals"}->{$tb}->{"sumF"}, $::EOL);
            print(CHECK "-------------------------------------------------",
                  "-----------", $::EOL);

            foreach my $prod (sort keys %::prodListAll) {
                my $prodEval = $prod . $pack;
                if ($id ne "") {
                    $prodEval .= "_" . $id;
                }
                my $lastData = "";
                LOOP_OS: foreach my $os (sort keys %::osListAll) {
                    # check for not tested combinations
                    foreach (@::noTestListAll) {
                        if ($_ eq "$os $tb $prod") {
                            if ($::debug) {
                                ::log("not tested: $os - $tb - $prod\n", 3);
                            }
                            next LOOP_OS;
                        }
                    }

                    my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
                    my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                    my $status = ::getStatus($statFile);
                    if ($status eq "") {
                        ::log("ERROR getting status for $os - $tb - $prod\n",
                              2);
                        $ret = 1;
                        next LOOP_OS;
                    } elsif ($status =~ /^NO/i or $status =~ /^-?S1( .*)?$/i) {
                        # there are no results yet for status '-S1' and 'S1'
                        next LOOP_OS;
                    }
                    # else: there are results for all other status codes
                    my $datFile = $EvalDir . $::sep . $prodEval . "." .
                                  $::FileExtResult;
                    my $dataLine = ::getDataLine($prod, $datFile);
                    my $outLine = $dataLine;
                    if ($status =~ /^S1E/i) {
                        substr($outLine, 3, 1) = "*";
                    } elsif ($status =~ /^-?S(\d)E?/i) {
                        my $postscanNr = $1 - 1;
                        substr($outLine, 3, 3) = "*p" . $postscanNr;
                    }

                    if ($lastData eq "") {
                        print(CHECK $::EOL);
                    }
                    print(CHECK $outLine,"  -  ",$os);
                    if ($lastData ne "" and $dataLine ne $lastData) {
                        print(CHECK " !!!");
                    }
                    print(CHECK $::EOL);
                    $lastData = $dataLine;
                }
            }
        }
    }
    return $ret;
}


# return a true value
1;