# Status.pm
# implements all AvtcTest commands that change the status of a reportfile
#
# v0.39, 2004-01-29
# Copyright (c) Michel Messerschmidt 2002-2004
#
package Avtc::Status;
use strict;
use Exporter;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;

@Avtc::Status::ISA = qw(Exporter);
@Avtc::Status::EXPORT = qw(showStatus reportStatus setOk setPostscan
                           resetOk resetPostscan changeStatus);


####################### implemented commands ###############################

# output status file to logfile/-window
sub showStatus {
    my $ret = 0;
    # status filenames are set in test configuration
    open(STAT, "< $::TestStatus") or do {
        ::log("ERROR: cannot open $::TestStatus: $!\n");
        return 1;
    };
    binmode(STAT);
    ::log("\nCurrent status for test '".$::configTestName."':\n", 0);
    my $line;
    while (defined ($line = <STAT>)) {
        $line = ::chommp($line);
        ::log($line."\n", 0);
    }
    close(STAT);
    return $ret;
}


# create/update status report file
sub reportStatus {
    my $ret = 0;
    # status filenames are set in test configuration
    open(RESULT, "> $::TestStatus") or do {
        close(RESULT);
        ::log("ERROR: cannot create $::TestStatus: $!\n");
        return 1;
    };
    binmode(RESULT);

    # write current date/time at the top of the status report
    ::log("writing to: $::TestStatus\n", 1);
    my $timestamp = localtime(time);
    print(RESULT "Datum: ", $timestamp, $::EOL, $::EOL);

    foreach my $os (sort keys %::osListAll) {
        ::log("processing operating system: $os\n", 1);
        print(RESULT " ",$os," status",$::EOL,"------------", $::EOL);
        # count number of products in same states
        my $prodNr = 0;
        my $doneNr = 0;
        my $needWorkNr = 0;
        my $beginNr = 0;
        my $noTestNr = 0;
        my %tempProds = ("Done" => "",
                        "Work" => "",
                        "Begin" => "",
                        "NoTest" => "");
        my $todoString = "Noch zu erledigen:" . $::EOL;
        ::log("processing product: ", 2);
        foreach my $prod (sort keys %::prodListAll) {
            my $infoDir = $::InfoRoot . $::sep . $prod;
            my $infoVer = $infoDir . $::sep . $::configDirNames{"infoVer"} .
                          $::sep . $::configTestName . "." . $os;
            my $infoOpt = $infoDir . $::sep . $::configDirNames{"infoOpt"} .
                           $::sep . $::configTestName . "." . $os;
            my $infoProb = $infoDir . $::sep . $::configDirNames{"infoProb"} .
                           $::sep . $::configTestName . "." . $os;
            my $infoProbAll = $infoDir . $::sep . $::configDirNames{"infoProb"} .
                           $::sep . $::configTestName . "." . "all";
            ::log("$prod ");
            $prodNr++;
            if (($prodNr % 8) == 0) {
                ::log("\n" . " "x24, 2);
            }
            my $prodNoTest = 0;
            my $prodDone = 0;
            my $prodWork = 0;
            my $prodBegin = 0;
            # list testbeds with work to do (per product)
            my %todo = ("S1" => "",
                        "S2" => "",
                        "S3" => "",
                        "E1" => "",
                        "E2" => "",
                        "E3" => "",
                        "Q1" => "",
                        "Q2" => "",
                        "Q3" => "",
                        "D" => "");
            LOOP_TB: foreach my $tb (sort keys %::listsAll) {
                # check for not tested combinations
                foreach my $skip (@::noTestListAll) {
                    if ($skip eq "$os $tb $prod") {
                        $prodNoTest = 1;
                        next LOOP_TB;
                    }
                }
                my $tbDir = $::listsAll{$tb}->{"id"};
                my $EvalDir = $::EvalBase .$::sep . $os . $::sep . $tb;
                my $NewLogDir = $::NewLogBase .$::sep . $os . $::sep .
                              $::listsAll{$tb}->{"id"};
                my $psl2 = $::FileExtScan2 . "." .$::FileExtPSL;
                my $psl3 = $::FileExtScan3 . "." .$::FileExtPSL;
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $status = ::getStatus($statFile);
                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod", 2);
                    $ret = 1;
                    next LOOP_TB;
                } elsif ($status =~ /^NO TEST( .*)?$/io) {
                    $prodNoTest = 1;
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^NO .+$/io) {
                    $prodDone = 1;
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^OK S\dEQ?/io) {
                    $prodDone = 1;
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S1E( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"Q1"} .= $tbDir.", ";
                    #$todo{"Q1"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S2E( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"Q2"} .= $tbDir.", ";
                    #$todo{"Q2"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S3E( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"Q3"} .= $tbDir.", ";
                    #$todo{"Q3"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S1( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"E1"} .= $tbDir.", ";
                    #$todo{"E1"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S2( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"E2"} .= $tbDir.", ";
                    #$todo{"E2"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^S3( .*)?$/io) {
                    $prodWork = 1;
                    # use this to report testbed ID instead of type
                    $todo{"E3"} .= $tbDir.", ";
                    #$todo{"E3"} .= $tb.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^-S1( .*)?$/io) {
                    $prodBegin = 1;
                    $todo{"S1"} .= $tbDir.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^-S2( .*)?$/io) {
                    $prodWork = 1;
                    $todo{"S2"} .= $tbDir.", ";
                    # copy postscan lists to 'scanlogs' dir
                    if (not -e $NewLogDir.$::sep.$prod.$psl2) {
                        copy($EvalDir.$::sep.$prod.$psl2,
                             $NewLogDir.$::sep.$prod.$psl2) or do {
                            ::log("ERROR copying ".$prod.$psl2.
                                  ": $!\n    ", 3, "\n");
                            $ret = 1;
                        };
                    }
                    if (-e $NewLogDir.$::sep.$prod.$psl3) {
                        unlink($NewLogDir.$::sep.$prod.$psl3);
                    }
                } elsif ($status =~ /^-S3( .*)?$/io) {
                    $prodWork = 1;
                    $todo{"S3"} .= $tbDir.", ";
                    if (-e $NewLogDir.$::sep.$prod.$psl2) {
                        unlink($NewLogDir.$::sep.$prod.$psl2);
                    }
                    # copy postscan lists to 'scanlogs' dir
                    if (not -e $NewLogDir.$::sep.$prod.$psl3) {
                        copy($EvalDir.$::sep.$prod.$psl3,
                             $NewLogDir.$::sep.$prod.$psl3) or do {
                            ::log("ERROR copying ".$prod.$psl3.
                                  ": $!\n    ", 3, "\n");
                            $ret = 1;
                        };
                    }
                }
            } #end of LOOP_TB

            # test for missing test documentation
            if ((not -e $infoVer) or -z $infoVer) {
                $prodWork = 1 unless ($prodNoTest or ($prodBegin and not $prodWork));
                $todo{"D"} .= "Version, ";
            }
            if ((not -e $infoOpt) or -z $infoOpt) {
                $prodWork = 1 unless ($prodNoTest or ($prodBegin and not $prodWork));
                $todo{"D"} .= "Optionen, ";
            }
            if ($todo{"D"} ne "" and ((not -e $infoProb and not -e $infoProbAll)
                                      or (-z $infoProb and -z $infoProbAll))) {
                $todo{"D"} .= "Probleme, ";
            }

            # assemble text strings
            # 'work' must be done first cause 'work' AND 'done' can be both set
            if ($prodDone and $prodBegin) {
                $prodWork = 1;
            }
            if ($prodNoTest and (not $prodDone)
                          and (not $prodWork)
                          and (not $prodBegin)) {
                # increase product counter for this state
                $noTestNr++;
                # and add this product to the list of products in this state
                $tempProds{"NoTest"} .= "$prod ";
            } elsif ($prodWork) {
                # increase product counter for this state
                $needWorkNr++;
                # add this product to the list of products in this state
                $tempProds{"Work"} .= "$prod ";

                foreach my $k (keys %todo) {
                    if ($todo{$k} ne "") {
                        $todoString .= $prod;
                        last;
                    }
                }
                # add a database list for each state of this product
                if ($todo{"S1"} ne "") {
                    # remove ", " at end of database list
                    chop($todo{"S1"}); chop($todo{"S1"});
                    my $temp = join(", ", sort split(", ", $todo{"S1"}));
                    $todoString .= " - Scan: " . $temp . $::EOL;
                }
                if ($todo{"S2"} ne "") {
                    chop($todo{"S2"}); chop($todo{"S2"});
                    my $temp = join(", ", sort split(", ", $todo{"S2"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Nachscan 1: " . $temp . $::EOL;
                    } else {
                        $todoString .= "    - Nachscan 1: " . $temp . $::EOL;
                    }
                }
                if ($todo{"S3"} ne "") {
                    chop($todo{"S3"}); chop($todo{"S3"});
                    my $temp = join(", ", sort split(", ", $todo{"S3"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Nachscan 2: " . $temp . $::EOL;
                    } else {
                        $todoString .= "    - Nachscan 2: " . $temp . $::EOL;
                    }
                }
                if ($todo{"E1"} ne "") {
                    chop($todo{"E1"});  chop($todo{"E1"});
                    my $temp = join(", ", sort split(", ", $todo{"E1"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Auswertung: " . $temp . $::EOL;
                    } else {
                        $todoString .= "    - Auswertung: " .$temp.$::EOL;
                    }
                }
                if ($todo{"E2"} ne "") {
                    chop($todo{"E2"});  chop($todo{"E2"});
                    my $temp = join(", ", sort split(", ", $todo{"E2"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Auswertung 2: " . $temp . $::EOL;
                    } else {
                        $todoString .= "    - Auswertung 2: " .$temp.$::EOL;
                    }
                }
                if ($todo{"E3"} ne "") {
                    chop($todo{"E3"});  chop($todo{"E3"});
                    my $temp = join(", ", sort split(", ", $todo{"E3"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Auswertung 3: " . $temp . $::EOL;
                    } else {
                        $todoString .= "    - Auswertung 3: " .$temp.$::EOL;
                    }
                }
                if ($todo{"Q1"} ne "") {
                    chop($todo{"Q1"});  chop($todo{"Q1"});
                    my $temp = join(", ", sort split(", ", $todo{"Q1"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Qualittssicherung: " . $temp .
                               $::EOL;
                    } else {
                        $todoString .= "    - Qualittssicherung: " .
                               $temp . $::EOL;
                    }
                }
                if ($todo{"Q2"} ne "") {
                    chop($todo{"Q2"});  chop($todo{"Q2"});
                    my $temp = join(", ", sort split(", ", $todo{"Q2"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Qualittssicherung 2: " . $temp .
                                       $::EOL;
                    } else {
                        $todoString .= "    - Qualittssicherung 2: " .
                                       $temp . $::EOL;
                    }
                }
                if ($todo{"Q3"} ne "") {
                    chop($todo{"Q3"});  chop($todo{"Q3"});
                    my $temp = join(", ", sort split(", ", $todo{"Q3"}));
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Qualittssicherung 3: " . $temp .
                                       $::EOL;
                    } else {
                        $todoString .= "    - Qualittssicherung 3: " .
                                       $temp . $::EOL;
                    }
                }
                if ($todo{"D"} ne "") {
                    chop($todo{"D"});  chop($todo{"D"});
                    if ($todoString =~ /^.*\n[A-Z0-9][A-Z0-9][A-Z0-9]$/os) {
                        $todoString .= " - Dokumentation: " . $todo{"D"} .
                                       $::EOL;
                    } else {
                        $todoString .= "    - Dokumentation: " . $todo{"D"} .
                                       $::EOL;
                    }
                }
            # now 'done' must be before 'begin', if they are both set
            # (if 'done' and 'begin' are set, 'work' is always set too)
            } elsif ($prodDone) {
                $doneNr++;
                $tempProds{"Done"} .= "$prod ";
            } elsif ($prodBegin) {
                $beginNr++;
                $tempProds{"Begin"} .= "$prod ";
                $todoString .= $prod . " - alles" . $::EOL;
            }
        }

        # remove last trailing whitespace
        chop($tempProds{"Done"}) if ($tempProds{"Done"});
        chop($tempProds{"Work"}) if ($tempProds{"Work"});
        chop($tempProds{"Begin"}) if ($tempProds{"Begin"});
        chop($tempProds{"NoTest"}) if ($tempProds{"NoTest"});

        # And finally write it all
        my $tempStr = sprintf("%3d nicht im Test (%s)", $noTestNr,
                              $tempProds{"NoTest"});
        my $noTestStr = ::indentString($tempStr, "(", 70);
        $tempStr = sprintf("%3d fertig (%s)", $doneNr,
                      $tempProds{"Done"});
        my $doneStr = ::indentString($tempStr, "(", 70);
        $tempStr = sprintf("%3d in Arbeit (%s)",
                      $needWorkNr, $tempProds{"Work"});
        my $workStr = ::indentString($tempStr, "(", 70);
        $tempStr = sprintf("%3d noch nicht getestet (%s)",
                       $beginNr, $tempProds{"Begin"});
        my $beginStr = ::indentString($tempStr, "(", 70);

        $todoString = ::indentString($todoString, ": ", 70);

        printf(RESULT "Von %d Produkten sind:%s", $prodNr, $::EOL);
        print(RESULT $noTestStr, $::EOL);
        print(RESULT $doneStr, $::EOL);
        print(RESULT $workStr, $::EOL);
        print(RESULT $beginStr, $::EOL, $::EOL);
        print(RESULT $todoString, $::EOL, $::EOL, $::EOL);
        ::log("\n");
    }

    close(RESULT);
    return $ret;
}



# change the status of a reportfile from 'evaluated' to 'OK'
# eg. from 'S2E' to 'OK S2E'
sub setOk {
    return changeStatus("OK");
}


# change the status of a reportfile from 'evaluated' to 'postscan'
# eg. from 'S1E' to '-S2'
sub setPostscan {
    return changeStatus("POSTSCAN");
}


# change the status of a reportfile from 'OK' to its previous state
# eg. from 'OK S1E' to 'S1E'
sub resetOk {
    return changeStatus("CUTOK");
}


# change the status of a reportfile from 'OK' to its previous state
# eg. from 'OK S1E' to 'S1E'
sub resetPostscan {
    return changeStatus("CUTPOSTSCAN");
}


# set status of all selected products to a new value
# parameter: new status
sub changeStatus {
    my $newStatus = $_[0];
    $newStatus = ::chommp(uc($newStatus));
    my $ret = 0;
    my $cutOk = 0;
    my $cutPostscan = 0;
    my $makeOk = 0;
    my $makePostscan = 0;
    # check special status codes
    # "CUTOK", "OK", "CUTPOSTSCAN" and "POSTSCAN" are never valid status
    # codes so they are used here for other purposes:
    # "CUTOK' is used to indicate that every status that is already finished
    # should have the "OK" prefix removed (reset to its previous status)
    # "CUTPOSTSCAN' is used to indicate that every postscan status should
    # be reset to its previous status
    # "OK" changes the status of a reportfile from 'evaluated' to 'OK',
    # eg. from 'S2E' to 'OK S2E'
    # "POSTSCAN" changes the status of a reportfile from 'evaluated' to
    # 'postscan', eg. from 'S1E' to '-S2'
    if ($newStatus =~ /^CUTOK$/i) {
        $cutOk = 1;
    } elsif ($newStatus =~ /^CUTPOSTSCAN$/i) {
        $cutPostscan = 1;
    } elsif ($newStatus =~ /^OK$/i) {
        $makeOk = 1;
    } elsif ($newStatus =~ /^POSTSCAN$/i) {
        $makePostscan = 1;
    }
    # insert whitespace if needed
    if ($newStatus =~ /^OK[^ ]+|^NO[^ ]+/i) {
        substr($newStatus, 2, 1) = " " . substr($newStatus, 2, 1);
    }
    if ($newStatus !~ /^NO |^-?S[1-3]$|^(OK )?S[1-3]E$/i and
        $newStatus !~ /^(CUTOK|OK|CUTPOSTSCAN|POSTSCAN)$/) {
        ::log("ERROR: illegal new status '$newStatus'\n", 1);
        return 1;
    }
    # ask for a name to store with the new status
    my $username = ::getUsername();

    foreach my $os (sort (keys %::osList)) {
        LOOP_TB: foreach my $tb (sort keys %::lists) {
            my $EvalDir = $::EvalBase . $::sep . $os . $::sep . $tb;
            my $LogDir = $::LogBase . $::sep . $os . $::sep . $tb;
            my $NewLogDir = $::NewLogBase . $::sep . $os . $::sep .
                            $::lists{$tb}->{"id"};
            my $TempDir = $::TempBase . $::sep . $os . $::sep . $tb;

            # check existance of TempDir and create it if nessecary
            ::makeDir($TempDir) or do {
                ::log("ERROR: creating directory $TempDir\n", 2);
            };

            LOOP_PROD: foreach my $prod (sort keys %::prodList) {
                # check for not tested combinations
                foreach (@::noTestListAll) {
                    if ($_ eq "$os $tb $prod") {
                        if ($::debug) {
                            ::log("    not tested: $os - $tb - $prod\n", 1);
                        }
                        next LOOP_PROD;
                    }
                }
                ::log("processing $prod - $tb - $os\n", 1) unless $::quiet;
                # define name of product related files
                my $statFile = $EvalDir . $::sep . $prod . $::FileExtStatus;
                my $ful = $prod . "." . $::FileExtReport;
                my $fuls1 = $prod . $::FileExtScan1 . "." . $::FileExtReport;
                my $fuls2 = $prod . $::FileExtScan2 . "." . $::FileExtReport;
                my $fuls3 = $prod . $::FileExtScan3 . "." . $::FileExtReport;
                my $txts2 = $prod . $::FileExtScan2 . "." . $::FileExtPSL;
                my $txts3 = $prod . $::FileExtScan3 . "." . $::FileExtPSL;
                my $archive = $LogDir . $::sep . $::LogArchiveName . "." .
                              $::ZipExt;
                my $oldArchive = $LogDir . $::sep . $::OldLogArchiveName .
                                 "." . $::ZipExt;
                my $status = ::getStatus($statFile);

                if ($status eq "") {
                    ::log("ERROR getting status for $os - $tb - $prod. ".
                          "No changes done\n", 2);
                    $ret = 1;
                    next LOOP_PROD;
                }
                # cut existing status or write a completely new status
                if ($cutOk == 1) {
                    if ($status =~ /^OK .+/i) {
                        my $out = substr($status, 3);
                        $out =~ s/^(S\dEQ?).*$/$1/i;
                        $out .= " (responsible: $username)";
                        ::updateStatus($statFile, $out) and do {
                            ::log("ERROR writing status file: $!\n", 2);
                            $ret = 1;
                            next LOOP_PROD;
                        };
                    } else {
                        ::log("ERROR can't execute 'reset ok' for a logfile".
                              " with status '".$status."'\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    }
                } elsif ($cutPostscan == 1) {
                    if ($status =~ /^-S(\d)/i) {
                        my $scanNr = $1;
                        # status '-S1' is no postscan!!!
                        if ($scanNr > 1) {
                            $scanNr -= 1;
                            my $out = "S" . $scanNr . "E";
                            $out .= " (responsible: $username)";
                            ::updateStatus($statFile, $out) and do {
                                ::log("ERROR writing status file: $!\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            };
                        } else {
                            ::log("ERROR can't execute 'reset postscan' for a".
                                  " logfile with "."status '".$status."'\n", 2);
                            $ret = 1;
                            next LOOP_PROD;
                        }
                    } else {
                        ::log("ERROR can't execute 'reset postscan' for a".
                              " logfile with "."status '".$status."'\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    }
                } elsif ($makeOk == 1) {
                    if ($status =~ /^S\dE/i) {
                        my $out = "OK " . $status;
                        $out .= " (responsible: $username)";
                        ::updateStatus($statFile, $out) and do {
                            ::log("ERROR writing status file: $!\n", 2);
                            $ret = 1;
                            next LOOP_PROD;
                        };
                    } else {
                        ::log("ERROR can't execute 'set ok' for a logfile ".
                              "with status '".$status."'\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    }
                } elsif ($makePostscan == 1) {
                    if ($status =~ /^S(\d)E/i) {
                        my $scan = $1;
                        if ($scan >= $::maxScan) {
                            ::log("ERROR: Maximum number of scans for this ".
                                  "reportfile is already reached.\n", 2);
                            ::log(" The status must be set to 'OK'\n", 5);
                            $ret = 1;
                            next LOOP_PROD;
                        }
                        $scan++;
                        my $out = "-S" . $scan;
                        $out .= " (responsible: $username)";
                        ::updateStatus($statFile, $out) and do {
                            ::log("ERROR writing status file: $!\n", 2);
                            $ret = 1;
                            next LOOP_PROD;
                        };
                    } else {
                        ::log("ERROR can't execute 'set postscan' for a ".
                              "logfile with status '".$status."'\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    }
                } else {
                    # remove previous names from old status string
                    # (only one of these substitutions is supposed to match)
                    $status =~ s/^(NO \S+)( .*)$/$1/i;
                    $status =~ s/^(OK S\dEQ?)( .*)$/$1/i;
                    $status =~ s/^(S\dEQ?)( .*)$/$1/i;
                    $status =~ s/^(-?S\d)( .*)$/$1/i;

                    # quick check, if there is really something to change
                    if ($newStatus eq $status) {
                        ::log("no status change required, skipping $prod\n", 2);
                        next LOOP_PROD;
                    }

                    # inspect all combinations of old and new status because not
                    # every combination makes sense. Status changes are only done
                    # for valid combinations.
                    # As a general rule it should not be possible to set a
                    # 'higher' status as before, eg. changing the status from
                    # '-S1' to 'OK S2E' must be prevented.
                    SWITCH: {
                        if ($newStatus =~ /^NO|^-S1$/i) {
                            if ($status !~ /^NO|^-S1/i) {
                                # delete evaluation data
                                deleteEvalFiles($prod, $EvalDir) or do {
                                    ::log("ERROR deleting files in ".
                                          $EvalDir."\n", 2);
                                    $ret = 1;
                                };
                                # delete reports from archive
                                deleteReport($archive, $oldArchive, $ful) or do {
                                    ::log("ERROR deleting $fuls2 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                                if ($status =~ /^(OK )?S[2-3]E?$/i) {
                                    deleteReport($archive, $oldArchive, $fuls2) or do {
                                        ::log("ERROR deleting $fuls2 from ".
                                                $archive."\n", 2);
                                        $ret = 1;
                                    };
                                }
                                if ($status =~ /^(OK )?S3E?$/i) {
                                    deleteReport($archive, $oldArchive, $fuls3) or do {
                                        ::log("ERROR deleting $fuls3 from ".
                                              $archive."\n", 2);
                                        $ret = 1;
                                    };
                                }
                                # delete postscan lists in evaldir
                                # (without error-checking)
                                unlink($EvalDir.$::sep.$txts2);
                                unlink($EvalDir.$::sep.$txts3);
                                # delete postscan lists in scanlogs
                                # (without error-checking)
                                unlink($NewLogDir.$::sep.$txts2);
                                unlink($NewLogDir.$::sep.$txts3);
                            }
                            last SWITCH;
                        }

                        if ($newStatus =~ /^OK S[1-3]E$/i) {
                            # setting an 'OK' status is only allowed if the
                            # current status is the last step before 'OK', eg.
                            # reset to 'OK S2E' is only allowed with current
                            # status 'S2E'
                            my $preStatus = substr($newStatus, 3);
                            if ($status ne $preStatus) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S1$/i) {
                            if ($status =~ /^NO|^-S1$/i) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # delete evaluation data
                            deleteEvalFiles($prod, $EvalDir) or do {
                                ::log("ERROR deleting files in $EvalDir\n", 2);
                                $ret = 1;
                            };
                            unlink($EvalDir.$::sep.$txts3);
                            unlink($EvalDir.$::sep.$txts2);

                            if ($status =~ /^-S2$/i) {
                                # delete postscan list (N1) in scanlogs
                                unlink($NewLogDir.$::sep.$txts2);
                            }
                            if ($status =~ /^(OK )?S2E?$/i) {
                                # delete xxxN1.ful from archive
                                deleteReport($archive, $oldArchive, $fuls2) or do {
                                    ::log("ERROR deleting $fuls2 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            if ($status =~ /^-S3$/i) {
                                # delete postscan list (N2) in scanlogs
                                unlink($NewLogDir.$::sep.$txts3);
                            }
                            if ($status =~ /^(OK )?S3E?$/i) {
                                # delete xxxN2.ful from archive
                                deleteReport($archive, $oldArchive, $fuls3) or do {
                                    ::log("ERROR deleting $fuls3 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S1E$/i) {
                            if ($status =~ /^NO|^-?S1$|^(OK )?S2E$|^(-|OK )?S3E?$/i) {
                                # resetting to an 'evaluated' status is only possible if no
                                # additional postscan reports were already evaluated
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            if ($status =~ /^-S2$/i) {
                                # delete postscan list (N1) in scanlogs
                                unlink($NewLogDir.$::sep.$txts2);
                            }
                            if ($status =~ /^S2$/i) {
                                # delete xxxN1.ful in archive
                                deleteReport($archive, $oldArchive, $fuls2) or do {
                                    ::log("ERROR deleting $fuls2 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            # 'OK S1E' can be changed without any additional steps
                            last SWITCH;
                        }

                        if ($newStatus =~ /^-S2$/i) {
                            if ($status =~ /^NO|^(-|OK )?S1E?$/i) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # delete evaluation data except postscan list (N1)
                            deleteEvalFiles($prod, $EvalDir) or do {
                                ::log("ERROR deleting files in $EvalDir\n", 2);
                                $ret = 1;
                            };
                            unlink($EvalDir.$::sep.$txts3);
                            # delete xxxN1.ful in archive
                            deleteReport($archive, $oldArchive, $fuls2) or do {
                                ::log("ERROR deleting $fuls2 from $archive".
                                      "\n", 2);
                                $ret = 1;
                            };

                            # copy postscan list (N1) to scanlogs
                            if ($status =~ /^-S3$/i) {
                                # delete postscan list (N2) in scanlogs
                                unlink($NewLogDir.$::sep.$txts3);
                            }
                            if ($status =~ /^(OK )?S3E?$/i) {
                                # delete xxxN2.ful from archive
                                deleteReport($archive, $oldArchive, $fuls3) or do {
                                    ::log("ERROR deleting $fuls3 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S2$/i) {
                            if ($status =~ /^NO|^(-|OK )?S1E?$|^-S2$/i) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # delete evaluation data except postscan list (N1)
                            deleteEvalFiles($prod, $EvalDir) or do {
                                ::log("ERROR deleting files in $EvalDir\n", 2);
                                $ret = 1;
                            };
                            unlink($EvalDir.$::sep.$txts3);

                            if ($status =~ /^-S3$/i) {
                                # delete postscan list (N2) in scanlogs
                                unlink($NewLogDir.$::sep.$txts3);
                            }
                            if ($status =~ /^(OK )?S3E?$/i) {
                                # delete xxxN2.ful from archive
                                deleteReport($archive, $oldArchive, $fuls3) or do {
                                    ::log("ERROR deleting $fuls3 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S2E$/i) {
                            if ($status !~ /^OK S2E$|^-?S3$/i) {
                                # resetting to an 'evaluated' status is only possible if no
                                # additional postscan reports were already evaluated
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            if ($status =~ /^-S3$/i) {
                                # delete postscan list (N2) in scanlogs
                                unlink($NewLogDir.$::sep.$txts3);
                            }
                            if ($status =~ /^S3$/i) {
                                # delete xxxN2.ful in archive
                                deleteReport($archive, $oldArchive, $fuls3) or do {
                                    ::log("ERROR deleting $fuls3 from ".
                                          $archive."\n", 2);
                                    $ret = 1;
                                };
                            }
                            # 'OK S2E' can be changed without any additional steps
                            last SWITCH;
                        }

                        if ($newStatus =~ /^-S3$/i) {
                            if ($status !~ /^(OK )?S3E?$/i) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # copy postscan list (N2) to scanlogs
                            copy($EvalDir.$::sep.$txts3, $NewLogDir.$::sep.$txts3) or do {
                                ::log("ERROR copying $txts3: $!\n", 2);
                                $ret = 1;
                            };
                            # delete evaluation data except postscan lists (N1 & N2)
                            deleteEvalFiles($prod, $EvalDir) or do {
                                ::log("ERROR deleting files in $EvalDir\n", 2);
                                $ret = 1;
                            };

                            # delete xxxN2.ful in archive
                            deleteReport($archive, $oldArchive, $fuls3) or do {
                                ::log("ERROR deleting $fuls3 from ".
                                      $archive."\n", 2);
                                $ret = 1;
                            };
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S3$/i) {
                            if ($status !~ /^(OK )?S3E$/i) {
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # delete evaluation data except postscan lists (N1 & N2)
                            deleteEvalFiles($prod, $EvalDir) or do {
                                ::log("ERROR deleting files in $EvalDir\n", 2);
                                $ret = 1;
                            };
                            last SWITCH;
                        }

                        if ($newStatus =~ /^S3E$/i) {
                            if ($status !~ /^OK S3E$/i) {
                                # resetting to an 'evaluated' status is only possible if no
                                # additional postscan reports were already evaluated
                                ::log("Illegal new status '$newStatus', ".
                                      "skipping $prod\n", 2);
                                $ret = 1;
                                next LOOP_PROD;
                            }
                            # 'OK S3E' can be changed without additional steps
                            last SWITCH;
                        }

                        # catch all remaining possibilities
                        ::log("ERROR: unknown new status '$newStatus' - ".
                              "skipping $prod\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    } # end of SWITCH
                    my $out = $newStatus . " (responsible: " . $username. ")";
                    ::updateStatus($statFile, $out) and do {
                        ::log("ERROR writing status file: $!\n", 2);
                        $ret = 1;
                        next LOOP_PROD;
                    };

                } # end of if-else-$cutOk
            }  #end of foreach prod LOOP_PROD
        }  #end of foreach tb
    }  #end of foreach os
    return $ret;
}


########################## helper functions #################################

# delete all files (except postscan lists) with a given prefix in a directory
sub deleteEvalFiles {
    my $prefix = $_[0];
    my $dir = $_[1];
    # these must not be deleted
    my $stat = uc($prefix . $::FileExtStatus);
    my $txts2 = uc($prefix . $::FileExtScan2 . "." . $::FileExtPSL);
    my $txts3 = uc($prefix . $::FileExtScan3 . "." . $::FileExtPSL);
    if ($::debug > 1) {
        ::log("deleting evaluation files\n", 3);
    }
    my @delFiles = ();
    opendir(DIR, $dir) or do {
        ::log("ERROR reading $dir\n", 3);
        ::log("!!! Can not delete files for $prefix !!!\n", 3);
        return 0;
    };
    my $filename;
    while (defined($filename = readdir(DIR))) {
        if ($filename =~ /^$prefix/i) {
            if (uc($filename) eq $txts2 or
                uc($filename) eq $stat or
                uc($filename) eq $txts3) {
                # don't delete postsacan lists
                if ($::debug > 2) {
                    ::log("skipped $filename\n", 4);
                }
                next;
            }
            push @delFiles, $dir . $::sep . $filename;
            if ($::debug > 2) {
                ::log("deleting ".$dir.$::sep.$filename."\n", 4);
            }
        }
    }
    closedir(DIR);
    my $delCount = unlink @delFiles;
    if ($::debug) {
        ::log("$delCount files deleted\n", 3);
    }
    if ($delCount < ($#delFiles + 1)) {
        ::log("ERROR deleting files: ". $#delFiles+1-$delCount .
              " files not deleted\n", 4);
        return 0;
    }
    return 1;
}


# delete a report files from an archive (and store a backup in backup archive).
# All archives must be given with full path, the file itself without any path.
sub deleteReport {
    my $archive = $_[0];
    my $bak = $_[1];
    my $file = $_[2];

    if ($::configOs eq $::OS_WINDOWS) {
        $archive = ::winPath($archive);
        $bak = ::winPath($bak);
    }

    my $oldDir = cwd();
    my $tempDir = $::TempBase . $::sep . "rep_bak" . $::sep;
    ::makeDir($tempDir) or do {
        ::log("ERROR: creating directory $tempDir\n", 2);
    };
    chdir($tempDir) or do {
        ::log("ERROR: chdir to $tempDir failed: $!\n", 3);
        return 0;
    };

    # extract file to delete / backup
    my $extCall = $::ZipProg ." ". $::UnzipOpt ." ". $archive ." ". $file;
    if ($::debug > 1) {
        ::log("calling '$extCall'\n", 4);
    }
    my $error = system($extCall);
    if ($::debug > 1) {
        ::log("...external program terminated with exitcode $error\n", 4);
    }
    if ($error) {
        ::log("ERROR calling $::ZipProg: exit code=$error, error=$!\n", 3);
        return 0;
    }

    # store file in backup archive
    $extCall = $::ZipProg . " " . $::ZipOpt . " " . $bak . " " . $file;
    if ($::debug > 1) {
        ::log("calling '$extCall'\n", 4);
    }
    $error = system($extCall);
    if ($::debug > 1) {
        ::log("...external program terminated with exitcode $error\n", 4);
    }
    if ($error) {
        ::log("ERROR calling $::ZipProg: exit code=$error, error=$!\n", 3);
        return 0;
    }

    # delete file from archive
    $extCall = $::ZipProg . " " . $::ZipDelOpt . " " . $archive . " " . $file;
    if ($::debug > 1) {
        ::log("calling '$extCall'\n", 4);
    }
    $error = system($extCall);
    if ($::debug > 1) {
        ::log("...external program terminated with exitcode $error\n", 4);
    }
    if ($error) {
        ::log("ERROR calling $::ZipProg: exit code=$error, error=$!\n", 3);
        return 0;
    }

    # delete temporary copy of file
    unlink($tempDir.$::sep.$file);

    chdir($oldDir) or do {
        ::log("ERROR: chdir back to $oldDir failed: $!\n", 4);
        return 0;
    };

    return 1;
}


# return a true value
1;
